Package: RStanTest
Type: Package
Title: What the Package Does (One Line, Title Case)
Version: 0.0.0.9000
Date: 2018-04-04
Authors@R: 
    person(given = "First",
           family = "Last",
           role = c("aut", "cre"),
           email = "first.last@example.com")
Description: What the package does (one paragraph).
License: What license it uses
Depends: R (>= 3.4.0)
Imports: Rcpp (>= 0.12.0), methods, rstan (>= 2.18.1), rstantools, RcppParallel (>= 5.0.1)
LinkingTo: StanHeaders (>= 2.18.0), rstan (>= 2.18.1), BH (>= 1.66.0), Rcpp (>= 0.12.0), RcppEigen (>= 0.3.3.3.0), RcppParallel (>= 5.0.1)
SystemRequirements: GNU make
Encoding: UTF-8
LazyData: true

Package: rstantools
Type: Package
Title: Tools for Developing R Packages Interfacing with 'Stan'
Version: 2.7.0
Date: 2026-07-25
Authors@R: 
    c(person(given = "Jonah",
             family = "Gabry",
             role = "aut",
             email = "jgabry@gmail.com"),
      person(given = "Ben",
             family = "Goodrich",
             role = "aut",
             email = "benjamin.goodrich@columbia.edu"),
      person(given = "Martin",
             family = "Lysy",
             role = "aut",
             email = "mlysy@uwaterloo.ca"),
      person(given = "Andrew",
             family = "Johnson",
             role = c("aut", "cre"),
             email = "andrew.johnson@arjohnsonau.com"),
      person(given = "Hamada S.",
             family = "Badr",
             role = "ctb"),
      person(given = "Marco",
             family = "Colombo",
             role = "ctb"),
      person(given = "Stefan",
             family = "Siegert",
             role = "ctb"),
      person(given = "Visruth",
             family = "Srimath Kandali",
             role = "ctb"),
      person(given = "Trustees of",
             family = "Columbia University",
             role = "cph"))
Description: Provides various tools for developers of R packages interfacing
    with 'Stan' <https://mc-stan.org>, including functions to set up the required
    package structure, S3 generics and default methods to unify function naming
    across 'Stan'-based R packages, and vignettes with recommendations for
    developers.
License: GPL (>= 3)
URL: https://mc-stan.org/rstantools/, https://discourse.mc-stan.org/
BugReports: https://github.com/stan-dev/rstantools/issues
Encoding: UTF-8
SystemRequirements: pandoc
Imports: desc, stats, utils, Rcpp (>= 0.12.16), RcppParallel (>=
        5.0.1), QuickJSR
Suggests: rstan (>= 2.17.2), usethis (>= 1.5.1), testthat (>= 2.0.0),
        knitr, pkgbuild, pkgload, roxygen2 (>= 7.0.0), rmarkdown,
        rstudioapi
VignetteBuilder: knitr
Config/roxygen2/version: 8.0.0
NeedsCompilation: no
Packaged: 2026-07-25 07:51:41 UTC; andrew
Author: Jonah Gabry [aut],
  Ben Goodrich [aut],
  Martin Lysy [aut],
  Andrew Johnson [aut, cre],
  Hamada S. Badr [ctb],
  Marco Colombo [ctb],
  Stefan Siegert [ctb],
  Visruth Srimath Kandali [ctb],
  Trustees of Columbia University [cph]
Maintainer: Andrew Johnson <andrew.johnson@arjohnsonau.com>
Repository: CRAN
Date/Publication: 2026-07-26 00:40:02 UTC
Built: R 4.5.2; ; 2026-07-26 01:07:05 UTC; unix

# Generated by roxygen2: do not edit by hand

S3method(bayes_R2,default)
S3method(loo_pit,default)
S3method(posterior_interval,default)
S3method(predictive_error,default)
S3method(predictive_interval,default)
S3method(prior_summary,default)
export(bayes_R2)
export(init_cpp)
export(log_lik)
export(loo_R2)
export(loo_epred)
export(loo_linpred)
export(loo_pit)
export(loo_predict)
export(loo_predictive_interval)
export(nsamples)
export(posterior_epred)
export(posterior_interval)
export(posterior_linpred)
export(posterior_predict)
export(predictive_error)
export(predictive_interval)
export(prior_summary)
export(rstan_config)
export(rstan_create_package)
export(rstantools_load_code)
export(use_rstan)
importFrom(RcppParallel,RcppParallelLibs)
importFrom(stats,quantile)
importFrom(stats,var)

#' The 'RSTAN_PACKAGE_NAME' package.
#'
#' @description A DESCRIPTION OF THE PACKAGE
#'
#' @docType package
#' @name RSTAN_PACKAGE_NAME-package
#' @aliases RSTAN_PACKAGE_NAME
#' @useDynLib RSTAN_PACKAGE_NAME, .registration = TRUE
#' @import methods
#' @import Rcpp
#' @importFrom rstan sampling
#' @importFrom rstantools rstan_config
#' @importFrom RcppParallel RcppParallelLibs
#'
#' @references
#' RSTAN_REFERENCE
#'
NULL

# names of stan models
stanmodels <- "STAN_MODEL_NAME"

# load each stan module
Rcpp::loadModule("stan_fit4STAN_MODEL_NAME_mod", what = TRUE)

# instantiate each stanmodel object
stanmodels <- sapply(stanmodels, function(model_name) {
  # create C++ code for stan model
  stan_file <- if(dir.exists("stan")) "stan" else file.path("inst", "stan")
  stan_file <- file.path(stan_file, paste0(model_name, ".stan"))
  stanfit <- rstan::stanc_builder(stan_file,
                                  allow_undefined = TRUE,
                                  obfuscate_model_name = FALSE)
  stanfit$model_cpp <- list(model_cppname = stanfit$model_name,
                            model_cppcode = stanfit$cppcode)
  # create stanmodel object
  methods::new(Class = "stanmodel",
               model_name = stanfit$model_name,
               model_code = stanfit$model_code,
               model_cpp = stanfit$model_cpp,
               mk_cppmodule = function(x) get(paste0("rstantools_model_", model_name)))
})

params <-
list(EVAL = TRUE)

## ----SETTINGS-knitr, include=FALSE--------------------------------------------
stopifnot(require(knitr))
opts_chunk$set(
  comment=NA,
  eval = if (isTRUE(exists("params"))) params$EVAL else FALSE
)
td <- tempdir()
PATH <- file.path(td, "rstanlm")
if(dir.exists(PATH)) {
  unlink(PATH, recursive = TRUE, force = TRUE)
}

## ----rstan_create_package, eval=FALSE-----------------------------------------
# library("rstantools")
# rstan_create_package(path = 'rstanlm')

## ----rstan_create_package-eval, echo=FALSE,warning=FALSE----------------------
library("rstantools")
rstan_create_package(path = PATH, rstudio=FALSE, open=FALSE)

## ----eval=FALSE---------------------------------------------------------------
# setwd("rstanlm")
# list.files(all.files = TRUE)

## ----echo=FALSE---------------------------------------------------------------
list.files(PATH, all.files = TRUE)

## ----eval=FALSE---------------------------------------------------------------
# file.show("DESCRIPTION")

## ----echo=FALSE---------------------------------------------------------------
DES <- readLines(file.path(PATH, "DESCRIPTION"))
cat(DES, sep = "\n")

## ----eval=FALSE---------------------------------------------------------------
# file.show("Read-and-delete-me")

## ----echo=FALSE---------------------------------------------------------------
cat(readLines(file.path(PATH, "Read-and-delete-me")), sep = "\n")

## ----eval=FALSE---------------------------------------------------------------
# file.remove('Read-and-delete-me')

## ----echo=FALSE---------------------------------------------------------------
file.remove(file.path(PATH, 'Read-and-delete-me'))

## ----include=FALSE------------------------------------------------------------
stan_prog <- "
data {
  int<lower=1> N;
  vector[N] x;
  vector[N] y;
}
parameters {
  real intercept;
  real beta;
  real<lower=0> sigma;
}
model {
  // ... priors, etc.

  y ~ normal(intercept + beta * x, sigma);
}
"
writeLines(stan_prog, con = file.path(PATH, "inst", "stan", "lm.stan"))
rstan_config(PATH)

## -----------------------------------------------------------------------------
# Save this file as `R/lm_stan.R`

#' Bayesian linear regression with Stan
#'
#' @export
#' @param x Numeric vector of input values.
#' @param y Numeric vector of output values.
#' @param ... Arguments passed to `rstan::sampling` (e.g. iter, chains).
#' @return An object of class `stanfit` returned by `rstan::sampling`
#'
lm_stan <- function(x, y, ...) {
  standata <- list(x = x, y = y, N = length(y))
  out <- rstan::sampling(stanmodels$lm, data = standata, ...)
  return(out)
}


## ----include=FALSE------------------------------------------------------------
Rcode <- "
#' Bayesian linear regression with Stan
#'
#' @export
#' @param x Numeric vector of input values.
#' @param y Numeric vector of output values.
#' @param ... Arguments passed to `rstan::sampling`.
#' @return An object of class `stanfit` returned by `rstan::sampling`
lm_stan <- function(x, y, ...) {
  out <- rstan::sampling(stanmodels$lm, data=list(x=x, y=y, N=length(y)), ...)
  return(out)
}
"
writeLines(Rcode, con = file.path(PATH, "R", "lm_stan.R"))

## ----eval=FALSE---------------------------------------------------------------
# file.show(file.path("R", "rstanlm-package.R"))

## ----echo=FALSE---------------------------------------------------------------
cat(readLines(file.path(PATH, "R", "rstanlm-package.R")), sep = "\n")

## ----eval = FALSE-------------------------------------------------------------
# try(roxygen2::roxygenize(load_code = rstantools_load_code), silent = TRUE)
# roxygen2::roxygenize()

## ----echo=FALSE, results="hide"-----------------------------------------------
try(roxygen2::roxygenize(PATH, load_code = rstantools_load_code), silent = TRUE)
roxygen2::roxygenize(PATH)

## ----eval=FALSE---------------------------------------------------------------
# # using ../rstanlm because already inside the rstanlm directory
# install.packages("../rstanlm", repos = NULL, type = "source")

## ----echo=FALSE---------------------------------------------------------------
install.packages(PATH, repos = NULL, type = "source")

## ----eval=FALSE---------------------------------------------------------------
# library("rstanlm")

## -----------------------------------------------------------------------------
fit <- lm_stan(y = rnorm(10), x = rnorm(10), 
               # arguments passed to sampling
               iter = 2000, refresh = 500)
print(fit)

## ----echo=FALSE---------------------------------------------------------------
unlink(PATH, recursive = TRUE, force = TRUE)

#' Generic function for pointwise log-likelihood
#'
#' We define a new function `log_lik()` rather than a
#' [stats::logLik()] method because (in addition to the conceptual
#' difference) the documentation for `logLik()` states that the return value
#' will be a single number, whereas `log_lik()` returns a matrix. See
#' the [log_lik.stanreg()](https://mc-stan.org/rstanarm/reference/log_lik.stanreg.html)
#' method in the \pkg{rstanarm} package for an example of defining a method.
#'
#' @export
#' @template args-object
#' @template args-dots
#'
#' @return `log_lik()` methods should return a \eqn{S} by \eqn{N} matrix,
#'   where \eqn{S} is the size of the posterior sample (the number of draws from
#'   the posterior distribution) and \eqn{N} is the number of data points.
#'
#' @template seealso-rstanarm-pkg
#' @template seealso-vignettes
#'
#' @examples
#' # See help("log_lik", package = "rstanarm")
#'
log_lik <- function(object, ...) {
  UseMethod("log_lik")
}

#' Generic function and default method for posterior uncertainty intervals
#'
#' These intervals are often referred to as credible intervals, but we use the
#' term uncertainty intervals to highlight the fact that wider intervals
#' correspond to greater uncertainty. See
#' [posterior_interval.stanreg()](https://mc-stan.org/rstanarm/reference/posterior_interval.stanreg.html)
#' in the \pkg{rstanarm} package for an example.
#'
#' @export
#' @template args-object
#' @template args-dots
#' @param prob A number \eqn{p \in (0,1)}{p (0 < p < 1)} indicating the desired
#'   probability mass to include in the intervals.
#'
#' @return `posterior_interval()` methods should return a matrix with two
#'   columns and as many rows as model parameters (or a subset of parameters
#'   specified by the user). For a given value of `prob`, \eqn{p}, the
#'   columns correspond to the lower and upper \eqn{100p}\% interval limits and
#'   have the names \eqn{100\alpha/2}\% and \eqn{100(1 - \alpha/2)}\%, where
#'   \eqn{\alpha = 1-p}. For example, if `prob=0.9` is specified (a
#'   \eqn{90}\% interval), then the column names would be `"5%"` and
#'   `"95%"`, respectively.
#'
#'   The default method just takes `object` to be a matrix (one column per
#'   parameter) and computes quantiles, with `prob` defaulting to `0.9`.
#'
#' @template seealso-rstanarm-pkg
#' @template seealso-vignettes
#'
#' @examples
#' # Default method takes a numeric matrix (of posterior draws)
#' draws <- matrix(rnorm(100 * 5), 100, 5) # fake draws
#' colnames(draws) <- paste0("theta_", 1:5)
#' posterior_interval(draws)
#'
#' # Also see help("posterior_interval", package = "rstanarm")
#'
posterior_interval <- function(object, ...) {
  UseMethod("posterior_interval")
}

#' @rdname posterior_interval
#' @export
posterior_interval.default <- function(object, prob = 0.9, ...) {
  if (!is.matrix(object))
    stop("For the default method 'object' should be a matrix.")
  .central_intervals(object, prob)
}

#' Generic function for drawing from the posterior predictive distribution
#'
#' Draw from the posterior predictive distribution of the outcome. See
#' [posterior_predict.stanreg()](https://mc-stan.org/rstanarm/reference/posterior_predict.stanreg.html)
#' in the \pkg{rstanarm} package for an example.
#'
#' @export
#' @template args-object
#' @template args-dots
#' @return `posterior_predict()` methods should return a \eqn{D} by \eqn{N}
#'   matrix, where \eqn{D} is the number of draws from the posterior predictive
#'   distribution and \eqn{N} is the number of data points being predicted per
#'   draw.
#'
#' @template seealso-rstanarm-pkg
#' @template seealso-vignettes
#'
#' @examples
#' # See help("posterior_predict", package = "rstanarm")
#'
posterior_predict <- function(object, ...) {
  UseMethod("posterior_predict")
}

#' Generic functions for LOO predictions
#'
#' See the methods in the \pkg{rstanarm} package for examples.
#'
#' @name loo-prediction
#'
#' @template args-object
#' @template args-dots
#'
#' @return `loo_predict()`, `loo_epred()`, `loo_linpred()`, and `loo_pit()`
#'   (probability integral transform) methods should return a vector with
#'   length equal to the number of observations in the data.
#'   For discrete observations, probability integral transform is randomised to
#'   ensure theoretical uniformity. Fix random seed for reproducible results 
#'   with discrete data. For more details, see Czado et al. (2009).
#'   `loo_predictive_interval()` methods should return a two-column matrix
#'   formatted in the same way as for [predictive_interval()].
#'
#' @template seealso-rstanarm-pkg
#' @template seealso-vignettes
#' @template reference-randomised-pit

#' @rdname loo-prediction
#' @export
loo_linpred <- function(object, ...) {
  UseMethod("loo_linpred")
}

#' @rdname loo-prediction
#' @export
loo_epred <- function(object, ...) {
  UseMethod("loo_epred")
}

#' @rdname loo-prediction
#' @export
loo_predict <- function(object, ...) {
  UseMethod("loo_predict")
}

#' @rdname loo-prediction
#' @export
loo_predictive_interval <- function(object, ...) {
  UseMethod("loo_predictive_interval")
}

#' @rdname loo-prediction
#' @export
loo_pit <- function(object, ...) {
  UseMethod("loo_pit")
}

#' @rdname loo-prediction
#' @export
#' @param y For the default method of `loo_pit()`, a vector of `y` values the
#'   same length as the number of columns in the matrix used as `object`.
#' @param lw For the default method of `loo_pit()`, a matrix of log-weights of
#'   the same length as the number of columns in the matrix used as `object`.
#'
loo_pit.default <- function(object, y, lw, ...) {
  if (!is.matrix(object))
    stop("For the default method 'object' should be a matrix.")
  stopifnot(
    is.numeric(object), is.numeric(y), length(y) == ncol(object),
    is.matrix(lw), identical(dim(lw), dim(object))
  )
  .loo_pit(y = y, yrep = object, lw = lw)
}

# internal ----------------------------------------------------------------
.loo_pit <- function(y, yrep, lw) {
  if (is.null(lw) || !all(is.finite(lw))) {
    stop("lw needs to be not null and finite.")
  }
  pits <- vapply(seq_len(ncol(yrep)), function(j) {
    sel_min <- yrep[, j] < y[j]
    pit <- .exp_log_sum_exp(lw[sel_min, j])
    sel_sup <- yrep[, j] == y[j]
    if (any(sel_sup)) {
      # randomized PIT for discrete y (see, e.g., Czado, C., Gneiting, T.,
      # Held, L.: Predictive model assessment for count data.
      # Biometrics 65(4), 1254–1261 (2009).)
      pit_sup <- pit + .exp_log_sum_exp(lw[sel_sup, j])
      pit <- stats::runif(1, pit, pit_sup)
    }
    pit
  }, FUN.VALUE = 1)
  if (any(pits > 1)) {
    warning(cat(
      "Some PIT values larger than 1! Largest: ",
      max(pits),
      "\nRounding PIT > 1 to 1."
    ))
  }
  pmin(1, pits)
}

.exp_log_sum_exp <- function(x) {
  m <- suppressWarnings(max(x))
  exp(m + log(sum(exp(x - m))))
}

# Part of the rstantools package
# Copyright (C) 2018, 2019 Martin Lysy
# Copyright (C) 2019 Trustees of Columbia University
#
# rstantools is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 3
# of the License, or (at your option) any later version.
#
# rstantools is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
#
#' Create a new \R package with compiled Stan programs
#'
#' @aliases rstan_package_skeleton
#'
#' @description
#'   \if{html}{\figure{stanlogo.png}{options: width="25" alt="https://mc-stan.org/about/logo/"}}
#'   The `rstan_create_package()` function helps get you started developing a
#'   new \R package that interfaces with Stan via the \pkg{rstan} package. First
#'   the basic package structure is set up via [usethis::create_package()].
#'   Then several adjustments are made so the package can include Stan programs
#'   that can be built into binary versions (i.e., pre-compiled Stan C++ code).
#'
#'   The **Details** section below describes the process and the
#'   **See Also** section provides links to recommendations for developers
#'   and a step-by-step walk-through.
#'
#'   As of version `2.0.0` of \pkg{rstantools} the
#'   `rstan_package_skeleton()` function is defunct and only
#'   `rstan_create_package()` is supported.
#'
#' @export
#' @param path The path to the new package to be created (terminating in the
#'   package name).
#' @param fields,rstudio,open Same as [usethis::create_package()]. See
#'   the documentation for that function, especially the note in the
#'   **Description** section about the side effect of changing the active
#'   project.
#' @param stan_files A character vector with paths to `.stan` files to include
#'   in the package.
#' @param roxygen Should \pkg{roxygen2} be used for documentation?  Defaults to
#'   `TRUE`. If so, a file `R/{pkgname}-package.R`` is added to the package with
#'   roxygen tags for the required import lines. See the **Note** section below
#'   for advice specific to the latest versions of \pkg{roxygen2}.
#' @param travis Should a `.travis.yml` file be added to the package directory?
#'   This argument is now deprecated. We recommend using GitHub Actions to set
#'   up automated testings for your package. See
#'   https://github.com/r-lib/actions for useful templates.
#' @template args-license
#' @template args-auto_config
#'
#' @details
#' This function first creates a regular \R package using
#' `usethis::create_package()`, then adds the infrastructure required to compile
#' and export `stanmodel` objects. In the package root directory, the user's
#' Stan source code is located in:
#' \preformatted{
#' inst/
#'   |_stan/
#'   |   |_include/
#'   |
#'   |_include/
#' }
#' All `.stan` files containing instructions to build a `stanmodel`
#' object must be placed in `inst/stan`.  Other `.stan` files go in
#' any `stan/` subdirectory, to be invoked by Stan's `#include`
#' mechanism, e.g.,
#' \preformatted{
#' #include "include/mylib.stan"
#' #include "data/preprocess.stan"
#' }
#' See \pkg{rstanarm} for many examples.
#'
#' The folder `inst/include` is for all user C++ files associated with the
#' Stan programs.  In this folder, the only file to directly interact with the
#' Stan C++ library is `stan_meta_header.hpp`; all other `#include`
#' directives must be channeled through here.
#'
#' The final step of the package creation is to invoke
#' [rstan_config()], which creates the following files for
#' interfacing with Stan objects from \R:
#' \itemize{
#'   \item `src` contains the `stan_ModelName{.cc/.hpp}` pairs
#'   associated with all `ModelName.stan` files in `inst/stan` which
#'   define `stanmodel` objects.
#'   \item `src/Makevars[.win]` which link to the `StanHeaders` and
#'   Boost (`BH`) libraries.
#'   \item `R/stanmodels.R` loads the C++ modules containing the
#'   `stanmodel` class definitions, and assigns an \R instance of each
#'   `stanmodel` object to a `stanmodels` list (with names
#'   corresponding to the names of the Stan files).
#' }
#' @template details-auto_config
#' @template details-license
#' @details Authors willing to license their Stan programs of general interest
#'   under the GPL are invited to contribute their `.stan` files and
#'   supporting \R code to the \pkg{rstanarm} package.
#'
#' @template section-running-stan
#'
#' @note For \pkg{devtools} users, because of changes in the latest versions of
#'   \pkg{roxygen2} it may be necessary to run `pkgbuild::compile_dll()`
#'   once before `devtools::document()` will work.
#'
#' @seealso
#' * [use_rstan()] for adding Stan functionality to an existing
#'   \R package and [rstan_config()] for updating an existing package
#'   when its Stan files are changed.
#' * The \pkg{rstanarm} package [repository](https://github.com/stan-dev/rstanarm)
#'   on GitHub.
#' @template seealso-vignettes
#' @template seealso-get-help
#' @template seealso-useR2016-video
#'
rstan_create_package <- function(path,
                                 fields = NULL,
                                 rstudio = TRUE,
                                 open = TRUE,
                                 stan_files = character(),
                                 roxygen = TRUE,
                                 travis = FALSE,
                                 license = TRUE,
                                 auto_config = TRUE) {
  if (!requireNamespace("usethis", quietly = TRUE)) {
    stop("Please install package 'usethis' to use function 'rstan_create_package'.",
         call. = FALSE)
  }
  DIR <- dirname(path)
  name <- basename(path)
  .check_stan_ext(stan_files)

  if (rstudio && !requireNamespace("rstudioapi", quietly = TRUE)) {
    stop("Please install package 'rstudioapi' to use option 'rstudio = TRUE'.",
         call. = FALSE)
    rstudio <- rstudio && rstudioapi::isAvailable()
  }
  if (open && rstudio) {
    on.exit(rstudioapi::openProject(file.path(DIR, name), newSession = TRUE))
  }

  # run usethis::create_package()
  if (file.exists(path)) {
    stop("Directory '", path, "' already exists.", call. = FALSE)
  }
  message("Creating package skeleton for package: ", name, domain = NA)
  suppressMessages(
    usethis::create_package(
      path = path,
      fields = fields,
      rstudio = rstudio,
      open = FALSE
    )
  )

  # add Stan-specific functionality to the new package
  pkgdir <- .check_pkgdir(file.path(DIR, name))
  .rstan_make_pkg(pkgdir, stan_files, roxygen, travis, license, auto_config)
  invisible(NULL)
}

#--- helper functions ----------------------------------------------------------

# check stan extensions
.check_stan_ext <- function(stan_files) {
  if (length(stan_files) && !all(grepl("\\.stan$", stan_files))) {
    stop("All files named in 'stan_files' must end ",
         "with a '.stan' extension.", call. = FALSE)
  }
}

# add travis file
.add_travis <- function(pkgdir) {
  travis_file <- readLines(.system_file("travis.yml"))
  .add_stanfile(gsub("RSTAN_PACKAGE_NAME", basename(pkgdir), travis_file),
                pkgdir, ".travis.yml",
                noedit = FALSE, msg = TRUE, warn = FALSE)
}

# add .gitignore and .Rbuildignore files
.add_gitignore_Rbuildignore <- function(pkgdir, travis) {
  gitignore_files <- c("^rcppExports.cpp$", "^stanExports_*")
  .add_stanfile(gitignore_files, pkgdir, ".gitignore",
                noedit = FALSE, msg = TRUE, warn = FALSE)

  Rbuildignore_files <- c(gitignore_files, if (travis) "^\\.travis\\.yml$")
  .add_stanfile(Rbuildignore_files, pkgdir, ".Rbuildignore",
                noedit = FALSE, msg = TRUE, warn = FALSE)
}

# add R/mypkg-package.R file with roxygen import comments
# also add Encoding: UTF-8 to DESCRIPTION
.add_roxygen <- function(pkgdir) {
  pkg_file <- readLines(.system_file("rstanpkg-package.R"))
  pkg_file <- gsub("RSTAN_PACKAGE_NAME", basename(pkgdir), pkg_file)
  pkg_file <- gsub("RSTAN_REFERENCE", .rstan_reference(), pkg_file)
  .add_stanfile(pkg_file, pkgdir,
                "R", paste0(basename(pkgdir), "-package.R"),
                noedit = FALSE, msg = TRUE, warn = FALSE)
  desc_pkg <- desc::description$new(file.path(pkgdir, "DESCRIPTION"))
  desc_pkg$set(Encoding = "UTF-8")
  desc_pkg$write()
}

# reference to rstan package
.rstan_reference <- function() {
  has_version <- utils::packageDescription("rstan", fields = "Version")
  version_year <- substr(utils::packageDescription("rstan", fields = "Date"), 1, 4)
  paste0(
    "Stan Development Team (", version_year,"). ",
    "RStan: the R interface to Stan. ",
    "R package version ", has_version, ". ",
    "https://mc-stan.org"
  )
}

# add stan functionality to package
.rstan_make_pkg <- function(pkgdir, stan_files,
                            roxygen, travis, license, auto_config) {

  use_rstan(pkgdir, license = license, auto_config = auto_config)
  file.copy(
    from = stan_files,
    to = file.path(pkgdir, "inst", "stan", basename(stan_files))
  )
  if (roxygen) .add_roxygen(pkgdir)
  if (travis) {
    .add_travis(pkgdir)
    warning(
      "The 'travis' argument is deprecated. ",
      "We recommend using GitHub Actions instead.",
      call. = FALSE
    )
  }
  .add_gitignore_Rbuildignore(pkgdir, travis)

  message("Configuring Stan compile and module export instructions ...")
  rstan_config(pkgdir)

  # open = "at" implies text is appended
  con <- file(file.path(pkgdir, "Read-and-delete-me"), open = "at")
  writeLines(readLines(.system_file("Read_and_delete_me")), con)
  close(con)

  message(
    domain = NA,
    sprintf(
      "Further Stan-specific steps are described in '%s'.",
      file.path(basename(pkgdir), "Read-and-delete-me")
    )
  )
}

#-------------------------------------------------------------------------------

## outline of steps:

## 1. run package skeleton.
##    - rm man files

## 2. create stan_dirs.
##    - inst/stan
##    - inst/stan/include
##    - inst/include
##    - src/stan_files
##    - if (!exists) message

## 3. add default files.
##    - src/stan_init.cpp
##    - inst/include/stan_meta_header.cpp

## 4. update NAMESPACE
##    - if (default) modify else message(remaining steps)

## 5. update DESCRIPTION
##    - if (modified) message else do_nothing

## 4. add stan_files

## 5. configure build
##    - make src/stan_files/*.{cc/hpp}
##      only overwrite if different
##    - add src/Makevars[.win]
##      only overwrite if different
##    - add R/stanmodels.R
##    - if (is_empty(inst/stan)) no Makevars, empty stanmodels.R

## messages:
##   - when creating directories
##   - when updating DESCRIPTION or NAMESPACE
##   - when adding files? sometimes

## warnings:
##   - when attempting to overwrite non-stan file with stan file of same name

## error:
##   - when {dir/file}.create fails even though it doesn't exist


## ok stan_meta_header.hpp is problematic, because want to warn if already exists, but only if it's there from before...

## so .add_stanfile(file_lines, pkgdir, ...,
##                  noedit = TRUE, msg = FALSE, warn = TRUE)

#--- test rstan_create_package -----------------------------------------------

# 1. create package under various conditions
# 2. pkgload::load_all package
# 3. run tests on package
# 4. delete package source

# setup
run_all_tests <- FALSE # if TRUE disables skip_on_cran and skip_on_travis
source("rstan_package_skeleton-testfunctions.R") # helper functions to run tests
pkg_name <- "RStanTest" # base name of package
# path to base directory where packages will be created
tmp_test_path <- TRUE # put all tests in temporary folder
if(tmp_test_path) test_path <- tempfile(pattern = "rstantools_")
dir.create(test_path, recursive = TRUE)
rand_path <- TRUE # randomize destination path
# path to package source files
pkg_src_path <- "RStanTest"
## pkg_src_path <- system.file("include", "RStanTest",
##                             package = "rstantools")
# package R files
code_files <- file.path(pkg_src_path, "postsamp.R")
# package stan files
stan_files <- file.path(pkg_src_path, c("SimpleModel.stan", "SimpleModel2.stan"))
# package stan/include files
incl_files <- file.path(pkg_src_path, "helper.stan")
# package C++ files
src_files <- file.path(pkg_src_path, "AddTest.cpp")

# various test conditions
test_descr <- expand.grid(
  ## create_package = c(TRUE, FALSE), # use create_package or package_skeleton
  create_package = TRUE, # package_skeleton now depreciated
  roxygen = c(TRUE, FALSE) # use roxygen for documentation
)
ntest <- nrow(test_descr)

# run tests
for(ii in 1:ntest) {
  if(rand_path) {
    pkg_dest_path <- file.path(test_path,
                               basename(tempfile(pattern = pkg_name)),
                               pkg_name)
  } else {
    pkg_dest_path <- file.path(test_path, pkg_name)
  }
  # specific test condition
  use_create_package <- test_descr$create_package[ii]
  use_roxygen <- test_descr$roxygen[ii]
  if (!use_roxygen) next # FIXME
  context(paste0(
    ifelse(use_create_package,
           yes = "rstan_create_package",
           no = "rstan_package_skeleton"),
    " -- ",
    ifelse(use_roxygen, "with", "without"),
    " roxygen")
    )
  # create package
  if(rand_path) dir.create(dirname(pkg_dest_path), recursive = TRUE)
  if(use_create_package) {
    # use rstan_create_package
    rstan_create_package(path = pkg_dest_path,
                         rstudio = FALSE, open = FALSE,
                         ## stan_files = stan_files,
                         roxygen = use_roxygen)
    # add R files
    file.copy(from = code_files,
              to = file.path(pkg_dest_path, "R", basename(code_files)))
  } else {
    stop("rstan_package_skeleton has been depreciated.")
    # use rstan_package_skeleton
    rstan_package_skeleton(name = pkg_name, path = dirname(pkg_dest_path),
                           ## stan_files = stan_files,
                           code_files = code_files,
                           roxygen = use_roxygen)
  }
  # add stan files (need to do this separately or else rstan_config complains)
  file.copy(from = stan_files,
            to = file.path(pkg_dest_path,
                           "inst", "stan", basename(stan_files)))
  # add stan/include files
  file.copy(from = incl_files,
            to = file.path(pkg_dest_path,
                           "inst", "stan", "include", basename(incl_files)))
  # add C++ files
  file.copy(from = src_files,
            to = file.path(pkg_dest_path, "src", basename(src_files)))
  rstan_config(pkg_dest_path)
  ## Rcpp::compileAttributes(pkg_dest_path)
  # install & load package
  test_that("Package loads correctly", {
    if(!run_all_tests) {
      skip_on_cran()
      skip_on_travis()
    }
    example(source) # defines the sourceDir() function
    try(roxygen2::roxygenize(pkg_dest_path, load_code = sourceDir), silent = TRUE)
    pkgbuild::compile_dll(pkg_dest_path)
    tmp <- capture.output(load_out <- pkgload::load_all(pkg_dest_path,
                                                 export_all = TRUE,
                                                 quiet = TRUE))
    expect_type(load_out, "list")
  })
  if(use_roxygen) {
    # check roxygen documentation
    # TODO: stop test if roxygen2 not found
    test_that("roxygen works properly", {
      if(!run_all_tests) {
        skip_on_cran()
        skip_on_travis()
      }
      skip_if_not_installed("roxygen2")
      ## devtools::document(pkg_dest_path)
      tmp <- capture.output(pkgload::unload(pkg_name))
      pkgbuild::compile_dll(pkg_dest_path)
      roxygen2::roxygenize(pkg_dest_path)
      tmp <- capture.output(load_out <- pkgload::load_all(pkg_dest_path,
                                                          export_all = TRUE,
                                                          quiet = TRUE))
      expect_identical(readLines(file.path(pkg_dest_path, "NAMESPACE")),
                       readLines(file.path(pkg_src_path, "NAMESPACE")))
    })
  }
  # check that functions work as expected
  test_that("logpost_R == logpost_Stan: postsamp1", {
    if(!run_all_tests) {
      skip_on_cran()
      skip_on_travis()
    }
    compare_postsamp1()
  })
  test_that("logpost_R == logpost_Stan: postsamp2", {
    if(!run_all_tests) {
      skip_on_cran()
      skip_on_travis()
    }
    compare_postsamp2()
  })
  test_that("external C++ code works", {
    if(!run_all_tests) {
      skip_on_cran()
      skip_on_travis()
    }
    n <- sample(1:20, 1)
    x <- rnorm(n)
    y <- rnorm(n)
    expect_equal(add_test(x, y), x + y)
  })
  # uninstall + delete package
  # tmp <- capture.output(pkgload::unload(pkg_name))
  ## detach(paste0("package:", pkg_name),
  ##        unload = TRUE, character.only = TRUE)
  ## remove.packages(pkgs = pkg_name, lib = lib_path) # remove installed package
  if(rand_path) {
    unlink(dirname(pkg_dest_path),
           recursive = TRUE, force = TRUE)
  } else {
    unlink(pkg_dest_path,
           recursive = TRUE, force = TRUE)
  }
}

# make sure everything gets deleted even if there are errors
teardown(code = {
  ## if(isNamespaceLoaded(pkg_name)) {
  ##   detach(paste0("package:", pkg_name),
  ##          unload = TRUE, character.only = TRUE)
  ## }
  if(tmp_test_path) unlink(test_path, recursive = TRUE, force = TRUE)
})

library(testthat)
library(rstantools)

foo <- function(x) {"test function"}

if (identical(Sys.getenv("TRAVIS"), "true")) {
  test_check("rstantools", filter = "methods")
} else {
  test_check("rstantools")
}

#' Generic function for extracting the number of posterior samples
#'
#' Extract the number of posterior samples stored in a fitted Bayesian model.
#'
#' @export
#' @template args-object
#' @template args-dots
#'
nsamples <- function(object, ...) {
  UseMethod("nsamples")
}

#' Tools for Developing R Packages Interfacing with Stan
#'
#' @name RStanTest
#' @docType package
#' @import Rcpp
#' @import methods
#' @importFrom rstan sampling
#' @useDynLib RStanTest, .registration = TRUE
NULL

#' Tools for Developing \R Packages Interfacing with Stan
#'
#' @name rstantools-package
#' @aliases rstantools
#' @docType package
#'
#' @description
#' \if{html}{
#'   \figure{stanlogo.png}{options: width="50" alt="mc-stan.org"}
#' } *Stan Development Team*
#'
#' The \pkg{rstantools} package provides various tools for developers of \R
#' packages interfacing with Stan (<https://mc-stan.org>), including
#' functions to set up the required package structure, S3 generic methods to
#' unify function naming across Stan-based \R packages, and vignettes with
#' guidelines for developers. To get started building a package see
#' [rstan_create_package()].
#'
#' @template seealso-vignettes
#' @template seealso-get-help
#' @template seealso-useR2016-video
#'
NULL


# internal ----------------------------------------------------------------

# release reminders (for devtools)
release_questions <- function() { # nocov start
  c(
    "Have you updated the developer guidelines in the vignette?"
  )
} # nocov end

#' Generic function and default method for Bayesian R-squared
#'
#' Generic function and default method for Bayesian version of R-squared for
#' regression models. A generic for LOO-adjusted R-squared is also provided. See
#' the [bayes_R2.stanreg()](https://mc-stan.org/rstanarm/reference/bayes_R2.stanreg.html)
#' method in the \pkg{rstanarm} package for an example of defining a method.
#'
#' @export
#' @template args-object
#' @template args-dots
#'
#' @return `bayes_R2()` and `loo_R2()` methods should return a vector of
#'   length equal to the posterior sample size.
#'
#'   The default `bayes_R2()` method just takes `object` to be a matrix of y-hat
#'   values (one column per observation, one row per posterior draw) and `y` to
#'   be a vector with length equal to `ncol(object)`.
#'
#'
#' @references
#' Andrew Gelman, Ben Goodrich, Jonah Gabry, and Aki Vehtari (2018). R-squared
#' for Bayesian regression models. *The American Statistician*, to appear.
#' DOI: 10.1080/00031305.2018.1549100.
#' ([Preprint](http://www.stat.columbia.edu/~gelman/research/published/bayes_R2_v3.pdf),
#' [Notebook](https://avehtari.github.io/bayes_R2/bayes_R2.html))
#'
#'
#' @template seealso-rstanarm-pkg
#' @template seealso-vignettes
#'
bayes_R2 <- function(object, ...) {
  UseMethod("bayes_R2")
}

#' @rdname bayes_R2
#' @export
#' @param y For the default method, a vector of `y` values the same length
#'   as the number of columns in the matrix used as `object`.
#'
#' @importFrom stats var
#'
bayes_R2.default <-
  function(object, y,
           ...) {
    if (!is.matrix(object))
      stop("For the default method 'object' should be a matrix.")
    stopifnot(NCOL(y) == 1, ncol(object) == length(y))
    ypred <- object
    e <- -1 * sweep(ypred, 2, y)
    var_ypred <- apply(ypred, 1, var)
    var_e <- apply(e, 1, var)
    var_ypred / (var_ypred + var_e)
  }


#' @rdname bayes_R2
#' @export
loo_R2 <- function(object, ...) {
  UseMethod("loo_R2")
}

#' Posterior Sampler for SimpleModel
#' @export
postsamp1 <- function(x, nsamples) {
  sampling(object = stanmodels$SimpleModel, data = list(x = x),
           iter = nsamples, chains = 1)
}

#' Posterior Sampler for SimpleModel2
#' @export
postsamp2 <- function(x, nsamples) {
  sampling(object = stanmodels$SimpleModel2, data = list(x = x),
           iter = nsamples, chains = 1)
}

## #' Posterior Sampler for SimpleModel3
## #' @export
## postsamp3 <- function(x, nsamples) {
##   sampling(object = stanmodels$SimpleModel3, data = list(x = x),
##            iter = nsamples, chains = 1)
## }

# run things silently
eval_quiet <- function(expr, no_warnings = FALSE) {
  if(no_warnings) {
    suppressMessages(suppressWarnings(expr))
  } else {
    suppressMessages(expr)
  }
}

eval_quiet(require(rstan)) # for log_prob & unconstrain_pars


#---------------------------------------------------------------------------

# return R and Stan log-posteriors for same parameter values

# R/Stan diff check for postsamp1
compare_postsamp1 <- function() {
  # simulate data
  x <- rnorm(1)
  logpost <- eval_quiet(postsamp1(x = x, nsamples = 1), no_warnings = TRUE)
  # parameter values
  ntest <- sample(20:30, 1)
  Pars <- replicate(n = ntest, expr = {
    list(mu = rnorm(1))
  }, simplify = FALSE)
  # log-posterior in R
  lpR <- sapply(1:ntest, function(ii) {
    mu <- Pars[[ii]]$mu
    dnorm(x, mu, 1, log = TRUE)
  })
  # log-posterior in Stan
  lpStan <- sapply(1:ntest, function(ii) {
    upars <- unconstrain_pars(object = logpost, pars = Pars[[ii]])
    log_prob(object = logpost, upars = upars, adjust_transform = FALSE)
  })
  # check that they differ by only a constant
  expect_equal(max(abs(diff(lpR-lpStan))), 0, tolerance = 1e-6)
}

# R/Stan diff check for postsamp2
compare_postsamp2 <- function() {
  # simulate data
  x <- rnorm(1)
  logpost <- eval_quiet(postsamp2(x = x, nsamples = 1), no_warnings = TRUE)
  # parameter values
  ntest <- sample(20:30, 1)
  Pars <- replicate(n = ntest, expr = {
    list(sigma = runif(1, .2, 4))
  }, simplify = FALSE)
  # log-posterior in R
  lpR <- sapply(1:ntest, function(ii) {
    sigma <- Pars[[ii]]$sigma
    dnorm(x, 0, sigma, log = TRUE)
  })
  # log-posterior in Stan
  lpStan <- sapply(1:ntest, function(ii) {
    upars <- unconstrain_pars(object = logpost, pars = Pars[[ii]])
    log_prob(object = logpost, upars = upars, adjust_transform = FALSE)
  })
  # check that they differ by only a constant
  expect_equal(max(abs(diff(lpR-lpStan))), 0, tolerance = 1e-6)
}

#-------------------------------------------------------------------------------

# check contents of src files
check_lines <- function(stan_file, pkg_src_path, pkg_dest_path) {
  sf <- sub("\\.stan$", "", basename(stan_file))
  for(ext in c(".cc", ".h")) {
    src_files <- rstantools:::.stan_prefix(sf, ext)
    src_files <- c(file.path(pkg_src_path, src_files),
                   file.path(pkg_dest_path, "src", src_files))
    # v3
    expect_known_output(object = cat(readLines(src_files[2]), sep = "\n"),
                        file = src_files[1])
    ## # v2
    ## expect_identical(readLines(src_files[1]), readLines(src_files[2]))
    ## # v1
    ## src_md5 <- tools::md5sum(src_files)
    ## names(src_md5) <- NULL
    ## expect_identical(src_md5[1], src_md5[2])
  }
}

# check modification times for src files
check_mtime <- function(stan_file, pkg_dest_path) {
  sf <- sub("\\.stan$", "", basename(stan_file))
  src_files <- rstantools:::.stan_prefix(sf, c(".cc", ".h"))
  file.mtime(file.path(pkg_dest_path, "src", src_files))
}

#-------------------------------------------------------------------------------

# unload, uninstall, and delete source for given package
remove_package <- function(pkg_name, pkg_path, lib_path,
                           unload = TRUE,
                           uninstall = TRUE,
                           unlink = TRUE) {
  if(unload && isNamespaceLoaded(pkg_name)) {
    # unload package from R session
    detach(paste0("package:", pkg_name),
           unload = TRUE, character.only = TRUE)
  }
  if(uninstall && dir.exists(file.path(lib_path, pkg_name))) {
    # uninstall package
    remove.packages(pkg_name, lib_path)
  }
  if(unlink && dir.exists(file.path(pkg_path, pkg_name))) {
    unlink(file.path(pkg_path, pkg_name), recursive = TRUE, force = TRUE)
  }
}

require(rstan) # for log_prob & unconstrain_pars

stan_quiet <- function(expr) {
  suppressMessages(suppressWarnings(expr))
}

context("postsamp")

test_that("postsamp1: logposterior", {
  # data
  x <- rnorm(1)
  logpost <- stan_quiet(postsamp1(x = x, nsamples = 1))
  # parameter values
  nsim <- sample(20:30, 1)
  Pars <- replicate(n = nsim, expr = {
    list(mu = rnorm(1))
  }, simplify = FALSE)
  # log-posterior in R
  lpR <- sapply(1:nsim, function(ii) {
    mu <- Pars[[ii]]$mu
    dnorm(x, mu, 1, log = TRUE)
  })
  # log-posterior in stan
  lpStan <- sapply(1:nsim, function(ii) {
    upars <- unconstrain_pars(object = logpost, pars = Pars[[ii]])
    log_prob(object = logpost, upars = upars, adjust_transform = FALSE)
  })
  expect_equal(max(abs(diff(lpR-lpStan))), 0, tolerance = 1e-6)
})

test_that("postsamp2: logposterior", {
  # data
  x <- rnorm(1)
  logpost <- stan_quiet(postsamp2(x = x, nsamples = 1))
  # parameter values
  nsim <- sample(20:30, 1)
  Pars <- replicate(n = nsim, expr = {
    list(sigma = runif(1, .2, 4))
  }, simplify = FALSE)
  # log-posterior in R
  lpR <- sapply(1:nsim, function(ii) {
    sigma <- Pars[[ii]]$sigma
    dnorm(x, 0, sigma, log = TRUE)
  })
  # log-posterior in stan
  lpStan <- sapply(1:nsim, function(ii) {
    upars <- unconstrain_pars(object = logpost, pars = Pars[[ii]])
    log_prob(object = logpost, upars = upars, adjust_transform = FALSE)
  })
  expect_equal(max(abs(diff(lpR-lpStan))), 0, tolerance = 1e-6)
})

# Part of the rstantools package
# Copyright (C) 2018 Martin Lysy
#
# rstantools is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 3
# of the License, or (at your option) any later version.
#
# rstantools is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
#

#' Add Stan infrastructure to an existing package
#'
#' Add Stan infrastructure to an existing \R package. To create a *new* package
#' containing Stan programs use [rstan_create_package()] instead.
#'
#' @template args-pkgdir
#' @template args-license
#' @template args-auto_config
#'
#' @details Prepares a package to compile and use Stan code by performing the
#'   following steps:
#' 1. Create `inst/stan` folder where all `.stan` files defining
#'   Stan models should be stored.
#' 1. Create `inst/stan/include` where optional `license.stan` file is stored.
#' 1. Create `inst/include/stan_meta_header.hpp` to include optional header
#'   files used by Stan code.
#' 1. Create `src` folder (if it doesn't exist) to contain the Stan C++ code.
#' 1. Create `R` folder (if it doesn't exist) to contain wrapper code to expose
#'   Stan C++ classes to \R.
#' 1. Update `DESCRIPTION` file to contain all needed dependencies to compile
#'   Stan C++ code.
#' 1. If `NAMESPACE` file is generic (i.e., created by [rstan_create_package()]),
#'   append `import(Rcpp, methods)`, `importFrom(rstan, sampling)`,
#'   and `useDynLib` directives.  If `NAMESPACE` is not generic, display message
#'   telling user what to add to `NAMESPACE` for themselves.
#'
#' @template details-auto_config
#' @template section-running-stan
#'
#' @return Invisibly, `TRUE` or `FALSE` indicating whether or not any files or
#'   folders where created or modified.
#'
#' @export
use_rstan <- function(pkgdir = ".", license = TRUE, auto_config = TRUE) {
  pkgdir <- .check_pkgdir(pkgdir)

  # stan folders
  acc <- .add_standir(pkgdir, "inst", "stan", "include",
                      msg = TRUE, warn = FALSE)
  acc <- acc | .add_standir(pkgdir, "inst", "include",
                            msg = TRUE, warn = FALSE)
  acc <- acc | .add_standir(pkgdir, "src",
                            msg = TRUE, warn = FALSE)
  acc <- acc | .add_standir(pkgdir, "R",
                            msg = TRUE, warn = FALSE)

  # add license.stan
  license <- .make_license(pkgdir, license)
  if (!is.null(license)) {
    .add_stanfile(license, pkgdir,
                  "inst", "stan", "include", "license.stan",
                  noedit = FALSE, msg = FALSE, warn = FALSE)
  }

  # add stan_meta_header.hpp
  meta_header <- .system_file("stan_meta_header.hpp")
  .add_stanfile(readLines(meta_header), pkgdir,
                "inst", "include", "stan_meta_header.hpp",
                noedit = FALSE, msg = FALSE, warn = FALSE)

  # add stan_init.cpp
  # not needed for compiling without src/stan_files subdirectory
  ## acc <- acc | .add_staninit(pkgdir)

  # add packages to DESCRIPTION file
  acc <- acc | .update_description(pkgdir,
                                   auto_config = auto_config, msg = TRUE)

  # add or remove configure files
  acc <- acc | any(.setup_configure(pkgdir, auto_config))

  # modify NAMESPACE, or output instructions for how to do so
  acc <- acc | .update_namespace(pkgdir, msg = TRUE)
  if (!acc) {
    message("Stan functionality is already enabled.")
  } else {
    message("Done.")
  }
  invisible(acc)
}

# make (or read) license file from logical/character specification
.make_license <- function(pkgdir, license) {
  if (is.logical(license) && !license) {
    # license == FALSE
    license <- NULL
  } else {
    if (is.logical(license)) {
      # default license
      license <- readLines(system.file("include", "sys", "lice_nse.stan",
                                       package = "rstantools"))
      license <- gsub("RSTAN_PACKAGE_NAME", basename(pkgdir), license)
    } else if (is.character(license)) {
      license <- readLines(license)
    } else {
      stop("license must be logical or character string.", call. = FALSE)
    }
  }
  license
}


# either updates the existing NAMESPACE if it is generic,
# or produces a message for needed modifications.
# msg: display message if NAMESPACE needs to be modified
# return value: whether or not NAMESPACE needed to be modified
.update_namespace <- function(pkgdir, msg = TRUE) {
  # read namespace file
  namespc <- file.path(pkgdir, "NAMESPACE")
  namespc <- readLines(namespc)
  # required namespace lines
  req_lines <- c("import(Rcpp)",
                 "import(methods)",
                 "importFrom(rstan, sampling)",
                 paste0("useDynLib(", basename(pkgdir),
                        ", .registration = TRUE)"))
  ## lib_req <- c(methods = NA, rcpp = NA, dynlib = NA)
  if (.is_generic_namespace(namespc)) {
    # automatically update default NAMESPACE
    if (msg) message("Updating NAMESPACE ...")
    writeLines(c("# Generated by roxygen2: fake comment so roxygen2 overwrites silently.",
                 namespc[!grepl("^#", namespc)],
                 req_lines),
                 con = file.path(pkgdir, "NAMESPACE"))
    acc <- TRUE # namespace was modified
  } else {
    # print message stating required dependencies
    # format existing namespace with no white space
    # and one directive per line
    whitespc <- "[ \n\t\r\v\f]*" # any white space
    namespc <- namespc[!grepl("^#", namespc)]
    namespc <- paste0(namespc, collapse = "\n")
    namespc <- gsub(whitespc, "", namespc)
    namespc <- paste0(strsplit(namespc, split = "[)]")[[1]], ")")
    # check for namespace dependencies
    msg_lines <- NULL
    acc <- FALSE
    if (!.has_import(namespc, pkg = "Rcpp")) {
      # import(Rcpp)
      msg_lines <- c(msg_lines, req_lines[1])
      acc <- TRUE
    }
    if (!.has_import(namespc, pkg = "methods")) {
      # import(methods)
      msg_lines <- c(msg_lines, req_lines[2])
      acc <- TRUE
    }
    if (!.has_import(namespc, pkg = "rstan", fun = "sampling")) {
      # importFrom(rstan, sampling)
      msg_lines <- c(msg_lines, req_lines[3])
      acc <- TRUE
    }
    dynlib <- paste0("^useDynLib[(]", basename(pkgdir),
                     ",[.]registration=TRUE[)]")
    if (!any(grepl(dynlib, namespc))) {
      # useDynLib(RSTAN_PACKAGE_NAME, .registration = TRUE)
      msg_lines <- c(msg_lines, req_lines[4])
      acc <- TRUE
    }
    if (acc && msg) {
      message("\nNext, add the following lines (e.g., via <package-name>-package.R if using roxygen) to your NAMESPACE:\n\n",
              paste0(msg_lines, collapse = "\n"), "\n")
    }
  }
  invisible(acc)
}

# whether or not namespc imports given pkg, or given fun from pkg
# assume namespc has been white space formatted
.has_import <- function(namespc, pkg, fun) {
  # check if whole package is imported
  import_pkg <- any(grepl("^import[(]", namespc) &
                    grepl(paste0("(,|[(])", pkg, "(,|[)])"), namespc))
  if (missing(fun)) {
    import_pkg
  } else {
    # check if given function is imported
    import_pkg || any(grepl(paste0("^importFrom[(]", pkg, ",", fun, "[)]"),
                            namespc))
  }
}

# check whether namespace is generic
.is_generic_namespace <- function(namespc) {
  # package.skeleton generic
  gen1 <- "exportPattern(\"^[[:alpha:]]+\")"
  # create_package generic
  gen2 <- c("# Generated by roxygen2: fake comment so roxygen2 overwrites silently.",
            "exportPattern(\"^[^\\\\.]\")")
  identical(namespc, gen1) || identical(namespc, gen2)
}

# add or remove configure files
.setup_configure <- function(pkgdir, add = TRUE) {
  conf_names <- c("configure", "configure.win")
  if (add) {
    acc <- sapply(conf_names, function(conf_name) {
      config <- readLines(.system_file(conf_name))
      .add_stanfile(config, pkgdir, conf_name,
                            noedit = TRUE, msg = FALSE, warn = TRUE)
    })
    if (any(acc)) message("Adding 'configure' files ...")
    # make scripts executable
    sapply(conf_names[acc], function(conf_name) {
      system(paste0('chmod +x "', file.path(pkgdir, conf_name), '"'))
    })
  } else {
    acc <- sapply(conf_names, function(conf_name) {
      noedit_msg <- .rstantools_noedit(conf_name)
      conf_name <- file.path(pkgdir, conf_name)
      if (file.exists(conf_name) &&
         (noedit_msg %in% readLines(conf_name, n = 5))) {
        file.remove(conf_name) # Stan file found.  remove it
      } else FALSE # no stan file found
    })
    if (any(acc)) message("Removing 'configure' files ...")
  }
  acc
}

#-------------------------------------------------------------------------------

# compile work flow
# 0.  optionally create package with package.skeleton.
#     delete man files so install.packages produces no errors.
# 1.  create necessary dirs.
#     don't need to warn if they already exist, because nothing happens
#     then anyways.  Add stan_meta_header.hpp and stan_init.cpp at this stage?
# 2.  update NAMESPACE.  if NAMESPACE doesn't exist or is default exportPattern,
#     add useDynLib and import(Rcpp, methods).
#     Otherwise, tell user what's left to be done.
# 3.  add stan_files if any.
# 4.  configure stan_files specific compile + export instructions, i.e.,
#     Makevars[.win] and stanmodels.R.  If no stan_files are present,
#     these files should be removed.

# question: what happens when no stan_files are present?
# i think stan_init.cpp should still be present.  otherwise, package doesn't
# compile if NAMESPACE has useDynLib.  how about stan_meta_header.cpp?
# let's say for now just have it.

# rstan_config
# Copyright (C) 2018 Martin Lysy
#
# rstantools is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 3
# of the License, or (at your option) any later version.
#
# rstantools is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
#

#' Configure system files for compiling Stan source code
#'
#' Creates or update package-specific system files to compile `.stan` model
#' files found in `inst/stan`.
#'
#' @export
#' @template args-pkgdir
#' @details The Stan source files for the package should be stored in:
#' \itemize{
#'   \item `inst/stan` for `.stan` files containing instructions to
#'   build a `stanmodel` object.
#'   \item `inst/stan/any_subfolder` for files to be included via the
#'   `#include "/my_subfolder/mylib.stan"` directive.
#'   \item `inst/stan/any_subfolder` for a `license.stan` file.
#'   \item `inst/include` for the `stan_meta_header.hpp` file, to be
#'   used for directly interacting with the Stan C++ libraries.
#' }
#'
#' @return Invisibly, whether or not any files were added/removed/modified by
#'   the function.
#'
rstan_config <- function(pkgdir = ".") {
  pkgdir <- .check_pkgdir(pkgdir) # check if package root directory
  # get stan model files
  stan_files <- list.files(file.path(pkgdir, "inst", "stan"),
                           full.names = TRUE,
                           pattern = "(\\.stan$)|(\\.stanfunctions$)")
  if (length(stan_files) != 0) {
    # add R & src folders in case run from configure[.win] script
    .add_standir(pkgdir, "R", msg = FALSE, warn = FALSE)
    .add_standir(pkgdir, "src", msg = FALSE, warn = FALSE)
    # convert all .stan files to .cc/.hpp pairs
    sapply(stan_files, .make_cc, pkgdir = pkgdir)
    # update package Makevars
    acc <- .setup_Makevars(pkgdir, add = TRUE)
    ## .add_Makevars(pkgdir)
    ## .add_Makevars(pkgdir, win = TRUE)
  } else {
    # no stan files, so get rid of Makevars
    acc <- .setup_Makevars(pkgdir, add = FALSE)
    ## mkv_files <- file.path(pkgdir, "src", c("Makevars", "Makevars.win"))
    ## mkv_files <- mkv_files[file.exists(mkv_files)]
    ## if(length(mkv_files) > 0) file.remove(mkv_files)
  }
  # remove any .cc/.hpp/.o triplets with no corresponding .stan file
  acc <- any(acc) | .rm_cc(pkgdir)
  # register exported modules as native routines
  Rcpp::compileAttributes(pkgdir)
  # update R/stanmodels.R with current set of models
  stanmodels <- .update_stanmodels(pkgdir)
  acc <- acc | .add_stanfile(stanmodels, pkgdir, "R", "stanmodels.R")
  invisible(acc)
}

#--- helper functions ----------------------------------------------------------

# prefix for stan C++ files
.stan_prefix <- function(..., start = FALSE) {
  paste0(ifelse(start, "^", ""), "stanExports_", ...)
}

# file basename without extension
.basename_noext <- function(file_names) {
  gsub(pattern = "(.*?)\\..*$",
       replacement = "\\1", basename(file_names))
}

# removes any Stan-generated files in src which don't correspond to a model in
# /inst/stan
.rm_cc <- function(pkgdir) {
  # stan model files
  stan_files <- list.files(file.path(pkgdir, "inst", "stan"),
                           full.names = FALSE, pattern = "*[.]stan$")
  # all files in src
  all_files <- list.files(file.path(pkgdir, "src"), full.names = FALSE)
  # reduce to stan model files
  src_files <- all_files[grepl("*[.](cc|h)$", all_files)]
  src_files <- src_files[grepl(.stan_prefix(start=TRUE), src_files)]
  # make sure 1st line is "don't edit"
  src_line1 <- sapply(file.path(pkgdir, "src", src_files), readLines, n = 1)
  src_files <- src_files[(src_line1 == .rstantools_noedit("foo.h")) |
                         (src_line1 == .rstantools_noedit("foo.cc"))]
  # stan model names corresponding to inactive stan files
  rm_names <- gsub(.stan_prefix(start=TRUE), "", src_files)
  rm_names <- unique(gsub("[.](cc|h)$", "", rm_names))
  rm_names <- rm_names[!(rm_names %in% gsub("[.]stan$", "", stan_files))]
  if (length(rm_names) > 0) {
    # get all cc/h/o files in src corresponding to these models
    rm_files <- c(outer(.stan_prefix(rm_names),
                        c(".cc", ".h", ".o"), paste0))
    # and finally the files to remove
    rm_files <- all_files[all_files %in% rm_files]
    acc <- file.remove(file.path(pkgdir, "src", rm_files))
    acc <- any(acc)
  } else {
    acc <- FALSE
  }

  acc
}


# creates Makevars[.win] file with Stan-specific compile instructions
# namely, location of StanHeaders and BH libraries,
# and subfolder compile instructions.
.add_Makevars <- function(pkgdir, win = FALSE) {
  mkv_name <- paste0("Makevars", if(win) ".win" else "")
  makevars <- readLines(.system_file(mkv_name))
  # replace generic line "SOURCES = ..." with package-specific line
  # not needed for compiling without src/stan_files subdirectory
  ## src_line <- list.files(file.path(pkgdir, "inst", "stan"),
  ##                        pattern = "*[.]stan$")
  ## if(length(src_line) > 0) {
  ##   src_line <- gsub("[.]stan$", ".cc", file.path("stan_files", src_line))
  ##   makevars[grep("^SOURCES", makevars)] <- paste0(c("SOURCES =", src_line),
  ##                                                  collapse = " ")
  ## }
  .add_stanfile(makevars, pkgdir, "src", mkv_name,
                noedit = TRUE, msg = FALSE, warn = TRUE)
  ## invisible(makevars)
}

# if add = TRUE, creates Makevars[.win] file with Stan-specific compile instructions.
# if add = FALSE, deletes these files if they have not been modified by user.
# return: whether or not file(s) were successfully added/removed
.setup_Makevars <- function(pkgdir, add = TRUE) {
  noedit_msg <- .rstantools_noedit("foo")
  if (add) {
    acc <- sapply(c("Makevars", "Makevars.win"), function(mkv_name) {
      makevars <- readLines(.system_file(mkv_name))
      .add_stanfile(makevars, pkgdir, "src", mkv_name,
                    noedit = TRUE, msg = FALSE, warn = TRUE)
    })
  } else {
    acc <- sapply(c("Makevars", "Makevars.win"), function(mkv_name) {
      noedit_msg <- .rstantools_noedit(mkv_name)
      mkv_name <- file.path(pkgdir, "src", mkv_name)
      if(file.exists(mkv_name) &&
         (readLines(mkv_name, n = 1) == noedit_msg)) {
        file.remove(mkv_name) # Stan file found.  remove it
      } else FALSE # no stan file found
    })
  }
  acc
}

# create .cc/.h pair from .stan file
# the .hpp file contains the C++ level class definition of the given stanmodel
# the .cc file contains the module definition which Rcpp uses to construct
# the corresponding R ReferenceClass.
# If the .stan file has a functions block but no parameters block, then there
# is no module definition but the functions are compiled and exported to the
# package's namespace.
.make_cc <- function(file_name, pkgdir) {
  model_name <- sub("[.]stan$", "", basename(file_name)) # model name
  ## path to src/stan_files
  ## stan_path <- file.path(pkgdir, "src", "stan_files")
  # create c++ code
  # .stanfunction compatibility only available after 2.29, so need
  # to manually wrap function definitions in functions { } before calling stanc
  if (grepl("\\.stanfunctions$", file_name) &&
      (utils::packageVersion('rstan') < 2.29)) {
    mod <- readLines(file_name)
    file_name <- paste0(.basename_noext(file_name), "_wrapped.stanfunctions")
    cat("functions {", mod, "}", sep = "\n",
        file = file_name)
  }
  stanc_ret <- rstan::stanc(file_name, allow_undefined = TRUE,
                            obfuscate_model_name = FALSE,
                            isystem = c(dirname(file_name), getwd(),
                                        file.path(pkgdir, "inst", "stan"),
                                        file.path(pkgdir, "inst", "include")))
  only_functions <- grepl("functions[[:space:]]*\\{",  stanc_ret$model_code) &
                    !grepl("data[[:space:]]*\\{", stanc_ret$model_code) &
                    !grepl("parameters[[:space:]]*\\{", stanc_ret$model_code) &
                    !grepl("transformed[[:space:]]parameters[[:space:]]*\\{", stanc_ret$model_code) &
                    !grepl("model[[:space:]]*\\{", stanc_ret$model_code) &
                    !grepl("generated[[:space:]]quantities[[:space:]]*\\{", stanc_ret$model_code)
  if (only_functions) {
    # file_name is a collection of Stan functions rather than a model
    cppcode <- rstan::expose_stan_functions(stanc_ret, dryRun = TRUE)
    cpp_lines <- scan(text = cppcode, what = character(),
                      sep = "\n", quiet = TRUE)
    cpp_lines <- cpp_lines[cpp_lines != "#include <exporter.h>"]

    # Stanc3 gives 'auto' return type for standalone functions, which
    #   causes errors with Rcpp::export, so need to replace the auto
    #   return with the plain type from the main definition
    if(utils::packageVersion('rstan') >= 2.26) {
      # Extract line numbers of functions to be exported
      decl_lines = grep("// \\[\\[Rcpp::export]]",cpp_lines) + 1

      # Replace auto return type with function plain type
      for(i in 1:length(decl_lines)) {
        cpp_lines[decl_lines[i]] <- .replace_auto(decl_lines[i],cppcode, cpp_lines)
      }
    }
    cat("#include <exporter.h>",
        "#include <stan/math/prim/mat/fun/Eigen.hpp>",
        "#include <stan/model/standalone_functions_header.hpp>",
        file = file.path(pkgdir, "src",
                         paste(basename(pkgdir), "types.h", sep = "_")),
        sep = "\n")

  } else { # actual Stan model
    cppcode <- stanc_ret$cppcode
    cppcode <- scan(text = cppcode, what = character(), sep = "\n", quiet = TRUE)
    class_declaration <- grep("^class[[:space:]]+[A-Za-z_]", cppcode)
    cppcode <- append(cppcode, values = "#include <stan_meta_header.hpp>",
                      after = class_declaration - 1L)
    # If stan model generated using stanc3, need to make sure that the rstan files are
    #   included after USE_STANC3 has been defined
    if(utils::packageVersion('rstan') >= 2.26) {
      stanc3_declaration <- grep("#define USE_STANC3", cppcode)
      cppcode <- append(cppcode, values = "#include <rstan/rstaninc.hpp>",
                        after = stanc3_declaration + 1)
    } else {
      cppcode <- c("#include <rstan/rstaninc.hpp>", cppcode)
    }
    # Stan header file
    hdr_name <- .stan_prefix(model_name, ".h")
    # get license file (if any)
    stan_license <- .read_license(dirname(file_name))
    .add_stanfile(file_lines = c(stan_license,
                                "#ifndef MODELS_HPP",
                                "#define MODELS_HPP",
                                "#define STAN__SERVICES__COMMAND_HPP",
                                cppcode, "#endif"),
                  pkgdir = pkgdir,
                  "src", hdr_name,
                  noedit = TRUE, msg = FALSE, warn = TRUE)
    # create Rcpp module exposing C++ class as R ReferenceClass
    suppressMessages({
      cpp_lines <-
        utils::capture.output(
                Rcpp::exposeClass(class = paste0("rstantools_model_", model_name),
                                  constructors = list(c("SEXP", "SEXP", "SEXP")),
                                  fields = character(),
                                  methods = c("call_sampler",
                                              "param_names",
                                              "param_names_oi",
                                              "param_fnames_oi",
                                              "param_dims",
                                              "param_dims_oi",
                                              "update_param_oi",
                                              "param_oi_tidx",
                                              "grad_log_prob",
                                              "log_prob",
                                              "unconstrain_pars",
                                              "constrain_pars",
                                              "num_pars_unconstrained",
                                              "unconstrained_param_names",
                                              "constrained_param_names",
                                              "standalone_gqs"),
                                  file = stdout(),
                                  header = paste0('#include "', hdr_name, '"'),
                                  module = paste0("stan_fit4",
                                                  model_name, "_mod"),
                                  CppClass = "rstan::stan_fit<stan_model, boost::random::ecuyer1988> ",
                                  Rfile = FALSE)
              )
    })
  }
  .add_stanfile(file_lines = cpp_lines,
                pkgdir = pkgdir,
                "src",
                ifelse(only_functions, paste0(model_name, ".cpp"),
                       .stan_prefix(model_name, ".cc")),
                noedit = TRUE, msg = FALSE, warn = TRUE)
  return(invisible(NULL))
}

# read license file (if any)
.read_license <- function(stan_path) {
  # look for any file named license.stan
  stan_license <- dir(stan_path,
                      pattern = "^license[.]stan$", recursive = TRUE,
                      ignore.case = TRUE,
                      full.names = TRUE)
  if (length(stan_license) > 1) {
    stop("Multiple license.stan files detected.", call. = FALSE)
  } else if(length(stan_license) == 0) {
    stan_license <- NULL
  } else {
    stan_license <- readLines(stan_license)
  }
  stan_license
}

# rewrites stanmodels.R reflecting current list of stan files
.update_stanmodels <- function(pkgdir) {
  model_names <- list.files(file.path(pkgdir, "inst", "stan"),
                            pattern = "*.stan$")
  only_functions <- sapply(model_names, FUN = function(nm) {
      lns <- readLines(file.path(pkgdir, "inst", "stan", nm))
      return(any(grepl("functions[[:space:]]*\\{" , lns)) &
            !any(grepl("data[[:space:]]*\\{", lns)) &
            !any(grepl("parameters[[:space:]]*\\{", lns)) &
            !any(grepl("transformed[[:space:]]parameters[[:space:]]*\\{", lns)) &
            !any(grepl("model[[:space:]]*\\{", lns)) &
            !any(grepl("generated[[:space:]]quantities[[:space:]]*\\{", lns)) )
    })
  model_names <- model_names[!only_functions]
  model_names <- gsub("[.]stan$", "", model_names)
  if (length(model_names) == 0) {
    stanmodels <- .rstantools_noedit("stanmodels.R")
  } else {
    stanmodels <- readLines(.system_file("stanmodels.R"))
    # lines for Rcpp::loadModule
    load_line <- grep("^# load each stan module$", stanmodels)
      load_module <- sapply(model_names, gsub,
                            pattern = "STAN_MODEL_NAME",
                            x = stanmodels[load_line+1],
                            USE.NAMES = FALSE)
    # line for stanmodels assignment
    model_names <- paste0("\"", model_names, "\"")
    model_names <- paste0(model_names, collapse = ", ")
    model_names <- paste0("c(", model_names, ")", collapse = "")
    model_line <- grep("^# names of stan models$", stanmodels)
      model_names <- gsub("\"STAN_MODEL_NAME\"", model_names,
                          stanmodels[model_line+1])
    # add new lines to stanmodels
    nlines <- length(stanmodels)
    stanmodels <- c(stanmodels[1:model_line],
                    model_names,
                    stanmodels[(model_line+2):load_line],
                    load_module,
                    stanmodels[(load_line+2):nlines])
  }
  stanmodels
}

# Replace auto return type in function exports with the plain type from the main body.
.replace_auto <- function(decl_line, cppcode, cpp_lines) {
  # Extract the name of function
  decl <- cpp_lines[decl_line]
  decl <- gsub("auto ","",decl,fixed=T)
  decl <- sub("\\(.*","",decl,perl=T)

  # Replace newlines with blank spaces
  t3 <- gsub("\n"," ",cppcode,fixed=T)

  # Identify code segment containing first declaration of function
  sf1 <- strsplit(t3,decl,fixed=T)[[1]][1]
  sf2 <- utils::tail(strsplit(sf1,";",fixed=T)[[1]],1)

  # Get location of type promotion (if present)
  promote_start <- regexec("stan::promote_args_t<",sf2)[[1]]

  if(promote_start > 0) {

    str_t <- strsplit(sf2,"")[[1]]
    promote_end <- promote_start + attr(promote_start,'match.length')

    count <- 1

    while(count > 0 & promote_end < length(str_t)) {
      count <- count + ifelse(str_t[promote_end] == "<", 1,
                             ifelse(str_t[promote_end] == ">", -1,0))
      promote_end <- promote_end + 1
    }

    sf2 <- paste0(c(str_t[1:(promote_start-1)], "double",
                   str_t[promote_end:length(str_t)]),
                 collapse = "")
  }

  # Extract return type declaration and replace promoted scalar
  #  type with double
  rtn_type <- gsub("template <typename(.*?)> ","",sf2)

  # Update model code with type declarations
  gsub("auto ", rtn_type, cpp_lines[decl_line],fixed=T)
}

#--- test rstan_config ---------------------------------------------

context("rstan_config")

# 0.  setup
# 1.  create empty package
# 2.  add stan files and (weakly) check them via expect_known_output.
# 3.  rerun config, check that file.mtimes are identical
# 4.  create fake .o files to mimick contents of src without compiling package.  also add other files to src.
# 5.  remove one stan file, rerun config, check that files are removed and other files are the same.
# 6.  delete package source

#--- 0.  setup -----------------------------------------------------------------

run_all_tests <- FALSE # if TRUE disables skip_on_cran and skip_on_travis
backtest_stan_cpp <- FALSE # if FALSE disables check of stan cc/h files
# against what is on record.  typically this test only fails
# when the user and package stan C++ library are out of sync.
# helper functions to run tests
source("rstan_package_skeleton-testfunctions.R")
pkg_name <- "RStanTest" # name of package
# path to directory where package will be created
tmp_test_path <- TRUE # put tests in temporary folder
if(tmp_test_path) test_path <- tempfile(pattern = "rstantools_")
dir.create(test_path, recursive = TRUE)
pkg_dest_path <- file.path(test_path, pkg_name)
# path to package source files
pkg_src_path <- "RStanTest"
## pkg_src_path <- system.file("include", "RStanTest",
##                             package = "rstantools")
# package R files
code_files <- file.path(pkg_src_path, "postsamp.R")
# package stan files
stan_files <- file.path(pkg_src_path, c("SimpleModel.stan", "SimpleModel2.stan"))
# package stan/include files
incl_files <- file.path(pkg_src_path, "helper.stan")

#--- 1.  create package --------------------------------------------------------

rstan_create_package(path = pkg_dest_path,
                     rstudio = FALSE, open = FALSE,
                     stan_files = character(), roxygen = FALSE,
                     license = FALSE, auto_config = FALSE)
# copy R files
file.copy(from = code_files,
          to = file.path(pkg_dest_path, "R", basename(code_files)))


#--- 2.  add stan files, check contents ----------------------------------------

# copy stan files
file.copy(from = stan_files,
          to = file.path(pkg_dest_path, "inst", "stan",
                         basename(stan_files)))
# copy stan/include files
file.copy(from = incl_files,
          to = file.path(pkg_dest_path, "inst", "stan", "include",
                         basename(incl_files)))


# add Stan C++ code
rstan_config(pkg_dest_path)

# check that Stan C++ source code lines match those on record
# this should only fail when output from rstan::stanc gets modified, e.g.,
# because stan version has changed.
# subsequent runs (in interactive mode) will pass as records get overwritten.
# R CMD check will not overwrite record: see testthat::expect_known_output
if(backtest_stan_cpp) {
  test_that("Stan src files are created properly", {
    if (!run_all_tests) {
      skip_on_cran()
      skip_on_travis()
    }
    for(sf in stan_files) {
      check_lines(sf,
                  pkg_src_path = pkg_src_path,
                  pkg_dest_path = pkg_dest_path)
    }
  })
}

#--- 3.  rerun config, check that file.mtimes are identical --------------------

test_that("Unmodified Stan src files are not overwritten", {
  if (!run_all_tests) {
    skip_on_cran()
    skip_on_travis()
  }
  pre_mtime <- lapply(stan_files, check_mtime, pkg_dest_path = pkg_dest_path)
  pre_mtime <- do.call(c, pre_mtime)
  rstan_config(pkg_dest_path)
  post_mtime <- lapply(stan_files, check_mtime, pkg_dest_path = pkg_dest_path)
  post_mtime <- do.call(c, post_mtime)
  expect_identical(pre_mtime, post_mtime)
})

#--- 4.  create fake .o files and other src files ------------------------------

# fake .o files
invisible(sapply(gsub("\\.stan$", "", basename(stan_files)), function(sf) {
  writeLines("fake .o file",
             con = file.path(pkg_dest_path, "src",
                             rstantools:::.stan_prefix(sf, ".o")))
}))

# dummy src files
src_files <- c("foo.cpp", "stanExports_foo.txt", "abcstanExports_foo.cc")
invisible(sapply(src_files, function(sf) {
  writeLines("// fake source file\n",
             con = file.path(pkg_dest_path, "src", sf))
}))


#--- 5.  remove stan file(s) rerun config --------------------------------------

test_that("src is properly updated after removing one inst/stan files", {
  if (!run_all_tests) {
    skip_on_cran()
    skip_on_travis()
  }

  # check modification time of all files in src
  pre_files <- list.files(file.path(pkg_dest_path, "src"),
                          full.names = TRUE)
  pre_mtime <- file.mtime(pre_files)
  # remove files associated with one stan file
  rm_file <- sample(basename(stan_files), 1) # file to remove
  file.remove(file.path(pkg_dest_path, "inst", "stan", rm_file))
  rstan_config(pkg_dest_path) # re-configure
  # calculate modification times
  post_files <- list.files(file.path(pkg_dest_path, "src"), full.names = TRUE)
  post_mtime <- file.mtime(post_files)
  # deduce which of pre_files should remain
  rm_file <- tools::file_path_sans_ext(rm_file)
  pre_filter <- !grepl(paste0("^stanExports_", rm_file, "\\.(cc|h|o)"),
                   basename(pre_files))
  expect_identical(pre_files[pre_filter], post_files)
  # now check modification times
  pre_filter <- pre_filter & (basename(pre_files) != "RcppExports.cpp")
  post_filter <- basename(post_files) != "RcppExports.cpp"
  expect_identical(pre_mtime[pre_filter], post_mtime[post_filter])
})

test_that("src is properly updated after removing all inst/stan files", {
  if (!run_all_tests) {
    skip_on_cran()
    skip_on_travis()
  }

  rm_files <- list.files(file.path(pkg_dest_path, "inst", "stan"),
                         pattern = "[.]stan$",
                         full.names = TRUE)
  file.remove(rm_files)
  rstan_config(pkg_dest_path) # re-configure
  curr_files <- sort(list.files(file.path(pkg_dest_path, "src"),
                                full.names = TRUE))
  exp_files <- sort(file.path(pkg_dest_path, "src", src_files))
  expect_identical(curr_files, exp_files)
})

#--- 6.  delete package source -------------------------------------------------

unlink(pkg_dest_path, recursive = TRUE, force = TRUE)

# make sure everything gets deleted even if there are errors
teardown(code = {
  if(tmp_test_path) unlink(test_path, recursive = TRUE, force = TRUE)
})


#--- scratch -------------------------------------------------------------------

## sf <- "stanExports_SimpleModel.h"
## setNames(c(tools::md5sum(file.path(pkg_dest_path, sf)),
##            tools::md5sum(file.path(pkg_dest_path, "src", sf)),
##            tools::md5sum(file.path(pkg_src_path, sf))), NULL)

#' Generic function for extracting information about prior distributions
#'
#' See [prior_summary.stanreg()](https://mc-stan.org/rstanarm/reference/prior_summary.stanreg.html)
#' in the \pkg{rstanarm} package for an example.
#'
#' @export
#' @template args-object
#' @template args-dots
#'
#' @return `prior_summary()` methods should return an object containing
#'   information about the prior distribution(s) used for the given model.
#'   The structure of this object will depend on the method.
#'
#'   The default method just returns `object$prior.info`, which is
#'   `NULL` if there is no `'prior.info'` element.
#'
#' @template seealso-rstanarm-pkg
#' @template seealso-vignettes
#'
#' @examples
#' # See help("prior_summary", package = "rstanarm")
#'
prior_summary <- function(object, ...) {
  UseMethod("prior_summary")
}

#' @rdname prior_summary
#' @export
prior_summary.default <- function(object, ...) {
  object$prior.info
}

#' Generic function for accessing the posterior distribution of the
#' conditional expectation
#'
#' Extract the posterior draws of the conditional expectation.
#' See the \pkg{rstanarm} package for an example.
#'
#' @export
#' @template args-object
#' @template args-dots
#' @return `posterior_epred()` methods should return a \eqn{D} by \eqn{N}
#'   matrix, where \eqn{D} is the number of draws from the posterior
#'   distribution distribution and \eqn{N} is the number of data points.
#'
#' @template seealso-rstanarm-pkg
#' @template seealso-vignettes
#'
posterior_epred <- function(object, ...) {
  UseMethod("posterior_epred")
}

# various helper functions for creating rstan-enabled packages

# comment to put on top of rstantools system files.
# deduce comment character based on file extension
.rstantools_noedit <- function(file_name) {
  # comment character
  file_name <- basename(file_name)
  ext <- gsub(".*[.]", "", file_name)
  if (ext == file_name) ext <- ""
  com <- ifelse(tolower(ext) %in% c("r", "", "win"), "#", "//")
  paste0(com, " ", "Generated by rstantools.  Do not edit by hand.")
}

# warning message not overwriting file
.warning_nowrite <- function(file_name) {
  warning("'", file_name,
          "' already exists.  Not overwritten by rstantools.",
          .call = FALSE)
  invisible(NULL)
}

# location of rstantools system files,
# i.e., templates to be copied to user's package
.system_file <- function(...) {
  system.file("include", "sys", ..., package = "rstantools")
}

# taken from Rcpp::compileAttributes
# validates package directory
.check_pkgdir <- function(pkgdir) {
  pkgdir <- normalizePath(pkgdir, winslash = "/")
  descFile <- file.path(pkgdir, "DESCRIPTION")
  if (!file.exists(descFile)) {
    stop("pkgdir must refer to the directory containing an R package.",
         call. = FALSE)
  }
  pkgdir
}

# create a stan directory
# msg: display message if attempt to create is made
# warn: warning if directory already exists
# return value: whether or not directory was created
.add_standir <- function(pkgdir, ..., msg = TRUE, warn = FALSE) {
  dir_name <- file.path(pkgdir, ...)
  acc <- FALSE
  if (file.exists(dir_name)) {
    if (!file.info(dir_name)$isdir) {
      stop(file.path(...), " exists but is not a directory.", call. = FALSE)
    }
  } else {
    if (msg) message("Creating ", file.path(...), " directory ...")
    acc <- dir.create(dir_name, recursive = TRUE,
                      showWarnings = TRUE)
    # this should not happen...
    if (!acc) stop(file.path(...), " directory not created.", call. = FALSE)
  }
  if (warn && !acc) .warning_nowrite(file.path(basename(pkgdir), ...))
  invisible(acc)
}

# add stan file consisting of character vector of file_lines
# * don't overwrite unless first line of destination file is .rstantools_noedit
# * only overwrite if file is not identical
# noedit: add "rstantools file: don't edit" to first line of file
# msg: display message if attempt to create is made
# warn: waning if non-stan file of same name already exists
# return value: whether or not file was successfully added
.add_stanfile <- function(file_lines, pkgdir, ...,
                          noedit = TRUE, msg = FALSE, warn = TRUE) {
  dest_file <- file.path(pkgdir, ...)
  noedit_msg <- .rstantools_noedit(dest_file)
  has_file <- file.exists(dest_file) # check if file exists
  # check if existing file is a stan file
  is_stanfile <- has_file && (noedit_msg %in% readLines(dest_file, n = 5))
  if (has_file && !is_stanfile) {
    # non-stan file found: don't overwrite
    if (warn) .warning_nowrite(file.path(basename(pkgdir), ...))
    acc <- FALSE
  }
  if (!has_file || is_stanfile) {
    if (noedit && (file_lines[1] != noedit_msg)) {
      # add "noedit" to top of file
      if (startsWith(file_lines[1], "#!")) {
        # in this case noedit_msg goes on 2nd line so file starts with #! interpreter line
        # see https://github.com/stan-dev/rstantools/issues/82
        file_lines <- c(file_lines[1], "", noedit_msg, "", file_lines[-1])
      } else {
        file_lines <- c(noedit_msg, "", file_lines)
      }
    }
    if (is_stanfile) {
      # stan file found: check if it needs to be overwritten
      old_lines <- readLines(dest_file)
      acc <- !identical(paste0(old_lines, collapse = "\n"),
                        paste0(file_lines, collapse = "\n"))
    } else acc <- TRUE
    if (acc) {
      # create/overwrite file
      if (msg) message("Adding ", basename(dest_file), " file ...")
      writeLines(file_lines, con = dest_file)
    }
  }
  invisible(acc)
}

#' Generic function for accessing the posterior distribution of the linear
#' predictor
#'
#' Extract the posterior draws of the linear predictor, possibly transformed by
#' the inverse-link function. See
#' [posterior_linpred.stanreg()](https://mc-stan.org/rstanarm/reference/posterior_linpred.stanreg.html)
#' in the \pkg{rstanarm} package for an example.
#'
#' @export
#' @template args-object
#' @template args-dots
#' @param transform Should the linear predictor be transformed using the
#'   inverse-link function? The default is `FALSE`, in which case the
#'   untransformed linear predictor is returned.
#' @return `posterior_linpred()` methods should return a \eqn{D} by \eqn{N}
#'   matrix, where \eqn{D} is the number of draws from the posterior
#'   distribution distribution and \eqn{N} is the number of data points.
#'
#' @template seealso-rstanarm-pkg
#' @template seealso-vignettes
#'
#' @examples
#' # See help("posterior_linpred", package = "rstanarm")
#'
posterior_linpred <- function(object, transform = FALSE, ...) {
  UseMethod("posterior_linpred")
}

# Part of the rstantools package
# Copyright (C) 2018 Trustees of Columbia University
#
# rstantools is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 3
# of the License, or (at your option) any later version.
#
# rstantools is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

#' Register functions implemented in C++
#'
#' If you set up your package using `rstan_package_skeleton()` before
#' version `1.2.1` of \pkg{rstantools} it may be necessary for you to call
#' this function yourself in order to pass `R CMD check` in \R
#' `>= 3.4`. If you used `rstan_package_skeleton()` in \pkg{rstantools} version
#' `1.2.1` or later then this has already been done automatically.
#'
#' @export
#' @param name The name of your package as a string.
#' @param path The path to the root directory for your package as a string. If
#'   not specified it is assumed that this is already the current working
#'   directory.
#' @return This function is only called for its side effect of writing the
#'   necessary `init.cpp` file to the package's `src/` directory.
#'
init_cpp <- function(name, path) {
  file <- file.path("src", "init.cpp")
  if (!missing(path))
    file <- file.path(path, file)

  writeLines(c(
    "// Generated by the rstantools package",
    "", "",
    "#include <R.h>",
    "#include <Rinternals.h>",
    "#include <R_ext/Rdynload.h>",
    "#include <R_ext/Visibility.h>",
    "#include <Rversion.h>",
    "", "",
    "static const R_CallMethodDef CallEntries[] = {",
    "  {NULL, NULL, 0}",
    "};",
    "", "",
    paste0("void attribute_visible R_init_", name, "(DllInfo *dll) {"),
    "  // next line is necessary to avoid a NOTE from R CMD check",
    "  R_registerRoutines(dll, NULL, CallEntries, NULL, NULL);",
    "  R_useDynamicSymbols(dll, TRUE); // necessary for .onLoad() to work",
    "}"
    ),
    con = file
  )
  return(invisible(NULL))
}

#' Generic function for predictive intervals
#'
#' See [predictive_interval.stanreg()](https://mc-stan.org/rstanarm/reference/predictive_interval.stanreg.html)
#' in the \pkg{rstanarm} package for an example.
#'
#' @export
#' @template args-object
#' @template args-dots
#' @param prob A number \eqn{p \in (0,1)}{p (0 < p < 1)} indicating the desired
#'   probability mass to include in the intervals.
#'
#' @return `predictive_interval()` methods should return a matrix with two
#'   columns and as many rows as data points being predicted. For a given value
#'   of `prob`, \eqn{p}, the columns correspond to the lower and upper
#'   \eqn{100p}\% interval limits and have the names \eqn{100\alpha/2}\% and
#'   \eqn{100(1 - \alpha/2)}\%, where \eqn{\alpha = 1-p}. For example, if
#'   `prob=0.9` is specified (a \eqn{90}\% interval), then the column names
#'   would be `"5%"` and `"95%"`, respectively.
#'
#'   The default method just takes `object` to be a matrix and computes
#'   quantiles, with `prob` defaulting to `0.9`.
#'
#' @template seealso-rstanarm-pkg
#' @template seealso-vignettes
#'
#' @examples
#' # Default method takes a numeric matrix (of draws from posterior
#' # predictive distribution)
#' ytilde <- matrix(rnorm(100 * 5, sd = 2), 100, 5) # fake draws
#' predictive_interval(ytilde, prob = 0.8)
#'
#' # Also see help("predictive_interval", package = "rstanarm")
#'
predictive_interval <- function(object, ...) {
  UseMethod("predictive_interval")
}

#' @rdname predictive_interval
#' @export
predictive_interval.default <- function(object, prob = 0.9, ...) {
  if (!is.matrix(object))
    stop("For the default method 'object' should be a matrix.")
  .central_intervals(object, prob)
}


# internal ----------------------------------------------------------------

#' Compute central intervals
#'
#' @noRd
#' @param object A numeric matrix
#' @param prob Probability mass to include in intervals (in (0,1))
#' @return See above.
#'
#' @importFrom stats quantile
.central_intervals <- function(object, prob) {
  stopifnot(is.matrix(object))
  if (length(prob) != 1L || prob <= 0 || prob >= 1)
    stop("'prob' should be a single number greater than 0 and less than 1.",
         call. = FALSE)
  alpha <- (1 - prob) / 2
  probs <- c(alpha, 1 - alpha)
  labs <- paste0(100 * probs, "%")
  out <- t(apply(object, 2, quantile, probs = probs))
  structure(out, dimnames = list(colnames(object), labs))
}

context("default methods")
set.seed(1111)
x <- matrix(rnorm(150), 50, 3)
y <- rnorm(ncol(x))

test_that("posterior_interval.default hasn't changed", {
  expect_equal_to_reference(
    posterior_interval(x, prob = 0.5),
    "posterior_interval.RDS"
  )
})
test_that("predictive_interval.default hasn't changed", {
  expect_equal_to_reference(
    predictive_interval(x, prob = 0.8),
    "predictive_interval.RDS"
  )
})
test_that("predictive_error.default works", {
  expect_equal_to_reference(
    predictive_error(x, y),
    "predictive_error.RDS"
  )
})
test_that("prior_summary.default works", {
  obj <- list(prior.info = "prior info")
  expect_identical(prior_summary(obj), obj[[1]])

  expect_null(prior_summary(list(abc = "prior_info")))
})
test_that("loo_pit.default works", {
  lw <- matrix(rnorm(150), 50, 3)
  expect_equal_to_reference(
    loo_pit(x, y, lw),
    "loo_pit.RDS"
  )
})
test_that("bayes_R2.default hasn't changed", {
  expect_equal_to_reference(
    bayes_R2(x, y),
    "bayes_R2.RDS"
  )
})

test_that("default methods throw correct errors", {
  expect_error(posterior_interval(1:10), "should be a matrix")
  expect_error(predictive_interval(1:10), "should be a matrix")
  expect_error(predictive_error(1:10, 1:10), "should be a matrix")
  expect_error(bayes_R2(1:10, 1:10), "should be a matrix")
  expect_error(bayes_R2(cbind(1:10, 1:10), 1:9),
               "ncol(object) == length(y) is not TRUE",
               fixed = TRUE)
})


# helper functions --------------------------------------------------------
test_that(".central_intervals returns correct structure", {
  a <- .central_intervals(x, prob = 0.5)
  expect_equal(dim(a), c(ncol(x), 2))
  expect_identical(colnames(a), c("25%", "75%"))
})
test_that("central_intervals throws errors", {
  err_msg <- "'prob' should be a single number greater than 0 and less than 1"
  expect_error(.central_intervals(x, prob = c(0.5, 0.25)), err_msg)
  expect_error(.central_intervals(x, prob = 0, err_msg))
  expect_error(.central_intervals(x, prob = 1, err_msg))
})
test_that(".central_intervals returns correct structure", {
  a <- .central_intervals(x, prob = 0.5)
  expect_equal(dim(a), c(ncol(x), 2))
  expect_identical(colnames(a), c("25%", "75%"))
})

test_that(".pred_errors returns correct structure", {
  err <- .pred_errors(x, y)
  expect_true(is.matrix(err))
  expect_equal(dim(err), dim(x))
})
test_that(".pred_errors throws errors", {
  expect_error(.pred_errors(x, y[-1]), "length(y) == ncol(object) is not TRUE",
               fixed = TRUE)
  expect_error(.pred_errors(x[,1], y), "is.matrix(object) is not TRUE",
               fixed = TRUE)
})

#' Update `DESCRIPTION` to include all packages required to compile Stan code.
#'
#' @noRd
#' @param pkgdir Package root directory.
#' @param auto_config Whether or not package should configure itself.  This
#'   means importing `rstantools` for linking via `configure[.win]`) and setting
#'   `Biarch: true` in the `DESCRIPTION`.
#' @param msg Whether or not to display a message if an attempt to update
#'   `DESCRIPTION` is made.
#'
#' @return Whether or not `DESCRIPTION` file was modified.
#'
#' @details Sanitizes the package dependencies as follows:
#' \itemize{
#'   \item If `rstantools` dependency version is lower, don't update
#'   existing dependency.
#'   \item If `rstantools` `Imports` dependency already exists in
#'   `Depends`, keep the `Depends` package but increase version if
#'   needed.
#' }
#'
#' Issues the following messages:
#' \itemize{
#'   \item If `auto_config = FALSE` and `rstantools` is imported.
#'   \item If `auto_config = FALSE` and `Biarch: true`.
#' }
#'
.update_description <- function(pkgdir, auto_config = FALSE, msg = TRUE) {
  # current desc file
  desc_pkg <- desc::description$new(file.path(pkgdir, "DESCRIPTION"))
  # save a copy
  desc_old <- desc::description$new(file.path(pkgdir, "DESCRIPTION"))
  # rstantools desc file
  desc_rstan <- desc::description$new(.system_file("DESCRIPTION"))
  # check for Biarch field
  has_biarch <- desc_pkg$has_fields("Biarch")
  if(!auto_config) {
    # check for rstantools dependence
    has_rstantools <- desc_pkg$has_dep("rstantools", type = "Imports") ||
      desc_pkg$has_dep("rstantools", type = "Depends")
    # check if Biarch: true
    has_biarch <- has_biarch && tolower(desc_pkg$get_field("Biarch")) == "true"
    # don't add dependence on rstantools
    desc_rstan$del_dep("rstantools", type = "Imports")
  } else {
    # use latest version of rstantools
    desc_rstan$set_dep("rstantools", type = "Imports",
                       version = paste0(">= ",
                                        utils::packageVersion("rstantools")))
    # add Biarch: true for multiarch Windows
    desc_pkg$set(Biarch = "true")
  }
  # update dependencies
  dep_pkg <- desc_pkg$get_deps()
  names(dep_pkg)[3] <- "vpkg"
  dep_rstan <- desc_rstan$get_deps()
  names(dep_rstan)[3] <- "vrstan"
  # convert all rstan Imports to Depends if they are in package Depends
  di <- dep_rstan$package %in% dep_pkg$package[dep_pkg$type == "Depends"]
  dep_rstan$type[di & dep_rstan$type == "Imports"] <- "Depends"
  # combine all dependencies
  dep_comb <- merge(x = dep_pkg, y = dep_rstan,
                    by = c("type", "package"), all = TRUE)
  # larger version of each package
  dep_comb <- cbind(dep_comb[1:2],
                    version = .version_max(dep_comb$vpkg, dep_comb$vrstan))
  desc_pkg$set_deps(dep_comb)
  # add additional elements from template DESCRIPTION file
  extra_fields <- c("SystemRequirements", "Encoding")
  do.call(desc_pkg$set, as.list(desc_rstan$get(extra_fields)))
  # check if description has changed
  acc <- !identical(desc_pkg$str(), desc_old$str())
  if(acc) {
    if(msg) message("Updating DESCRIPTION ...")
    desc_pkg$write()
  }
  if(!auto_config && msg) {
    if(has_rstantools) {
      message("'Import: rstantools' detected in DESCRIPTION with 'auto_config = FALSE'.\n",
              "If you did not add this dependence yourself then it's safe to remove it.\n")
    }
    if(has_biarch) {
      message("'Biarch: true' detected in DESCRIPTION with 'auto_config = FALSE'.\n",
              "If you did not add this line yourself then it's safe to remove it.\n")
    }
  }
  invisible(acc)
}

# return greater of two version numbers in desc::desc_get_dep format
.version_max <- function(ver1, ver2) {
    # convert non-existing/any version packages to 0
    ver1 <- gsub("^[^[:digit:]]*([[:digit:]]*)",
                 replacement = "\\1", ver1)
    ver1[is.na(ver1) | ver1 == ""] <- 0
    ver2 <- gsub("^[^[:digit:]]*([[:digit:]]*)",
                 replacement = "\\1", ver2)
    ver2[is.na(ver2) | ver2 == ""] <- 0
    # take greater of two versions
    vmax <- mapply(function(x, y) utils::compareVersion(x, y) >= 0,
                   x = ver1, y = ver2)
    vmax <- ifelse(vmax, ver1, ver2)
    ifelse(vmax == "0", "*", paste0(">= ", vmax))
}

# walkthrough for using package skeleton

require(rstantools)

# change directory to where this file is located
pkg_path <- getwd()

#--- step 0: create a package without Stan code --------------------------------

# if you want to create an entire package with Stan code from scratch,
# skip to step 5

pkg_name <- "RStanTest"
package.skeleton(name = pkg_name,
                 code_files = "postsamp.R")
# delete contents of "man" and NAMESPACE so roxygen2 can be used
file.remove(list.files(file.path(pkg_path, pkg_name, "man"),
                       full.names = TRUE))
file.remove(file.path(pkg_path, pkg_name, "NAMESPACE"))

# add testthat tests
usethis::use_testthat(pkg_name)
file.copy(from = "test-postsamp.R",
          to = file.path(pkg_name, "tests", "testthat", "test-postsamp.R"))

# setup documentation with roxygen2
roxygen2::roxygenize(pkg_name)

#--- Step 1: add Stan functionality to package ---------------------------------

# this sets up the package to do this
rstantools::use_rstan(pkg_name)

# follow the instructions to add to the NAMESPACE
# since we're using roxygen2, the right way to do this is to
# add the lines to an R file in the package.
# I like to make this the R/RStanTest-package.R file

file.copy(from = file.path(pkg_path, "RStanTest-package.R"),
          to = file.path(pkg_path, pkg_name, "R", "RStanTest-package.R"))

#--- Step 2: add Stan files to package -----------------------------------------

# copy stan_files to inst/stan
stan_files <- c("SimpleModel.stan", "SimpleModel2.stan")
file.copy(from = file.path(pkg_path, stan_files),
          to = file.path(pkg_path, pkg_name, "inst", "stan", stan_files))

# configure package to create C++ code associated with stan_files
# ***NOTE*** You MUST run this every time you
# modify/add/delete anything in inst/stan
rstantools::rstan_config(pkg_name)


# re-roxygen2, which will automatically compile package
roxygen2::roxygenize(pkg_name)

#--- step 4: install package and run tests -------------------------------------

utils::install.packages(pkg_name, repos = NULL, type = "source")

# quit and restart, then:
testthat::test_package("RStanTest", reporter = "progress")


#--- step 5: create an entire package with Stan code from scratch --------------

pkg_path <- getwd()
pkg_name <- "RStanTest"
code_files <- c("RStanTest-package.R", "postsamp.R")
stan_files <- c("SimpleModel.stan", "SimpleModel2.stan")

rstantools::rstan_package_skeleton_plus(name = pkg_name,
                                        path = pkg_path,
                                        code_files = code_files,
                                        stan_files = stan_files)
# add tests
usethis::use_testthat(pkg_name)
file.copy(from = "test-postsamp.R",
          to = file.path(pkg_name, "tests", "testthat", "test-postsamp.R"))

# roxygen2 can't overwrite NAMESPACE unless it created it,
# so trick it into thinking it did
# (and add the rstan stuff or else you get errors...)
writeLines(c("# Generated by roxygen2: do not edit by hand",
             "",
             "import(Rcpp,methods)",
             "useDynLib(RStanTest, .registration = TRUE)"),
           con = file.path(pkg_path, pkg_name, "NAMESPACE"))

roxygen2::roxygenize(pkg_name) # run roxygen2
utils::install.packages(pkg_name, repos = NULL, type = "source")

# quit and restart, then:
testthat::test_package("RStanTest", reporter = "progress")

#' Generic function and default method for predictive errors
#'
#' Generic function and default method for computing predictive errors
#' \eqn{y - y^{rep}}{y - yrep} (in-sample, for observed \eqn{y}) or
#' \eqn{y - \tilde{y}}{y - ytilde} (out-of-sample, for new or held-out \eqn{y}).
#' See [predictive_error.stanreg()](https://mc-stan.org/rstanarm/reference/predictive_error.stanreg.html)
#' in the \pkg{rstanarm} package for an example.
#'
#' @export
#' @template args-object
#' @template args-dots
#' @return `predictive_error()` methods should return a \eqn{D} by \eqn{N}
#'   matrix, where \eqn{D} is the number of draws from the posterior predictive
#'   distribution and \eqn{N} is the number of data points being predicted per
#'   draw.
#'
#'   The default method just takes `object` to be a matrix and `y` to be a
#'   vector.
#'
#' @template seealso-rstanarm-pkg
#' @template seealso-vignettes
#'
#' @examples
#' # default method
#' y <- rnorm(10)
#' ypred <- matrix(rnorm(500), 50, 10)
#' pred_errors <- predictive_error(ypred, y)
#' dim(pred_errors)
#' head(pred_errors)
#'
#' # Also see help("predictive_error", package = "rstanarm")
#'
predictive_error <- function(object, ...) {
  UseMethod("predictive_error")
}

#' @rdname predictive_error
#' @export
#' @param y For the default method, a vector of `y` values the same length as
#'   the number of columns in the matrix used as `object`.
predictive_error.default <- function(object, y, ...) {
  if (!is.matrix(object))
    stop("For the default method 'object' should be a matrix.")
  .pred_errors(object, y)
}


# internal ----------------------------------------------------------------

# @param object A matrix
# @param y A vector the same length as ncol(object)
.pred_errors <- function(object, y) {
  stopifnot(is.matrix(object), length(y) == ncol(object))
  sweep(-1 * object, MARGIN = 2, STATS = as.array(y), FUN = "+")
}

context("generics with no methods")
set.seed(1111)
x <- matrix(rnorm(150), 50, 3)
y <- rnorm(ncol(x))


generics_w_no_default <-
  c(
    "log_lik",
    "posterior_predict",
    "posterior_linpred",
    "loo_predict",
    "loo_linpred",
    "loo_predictive_interval"
  )

test_that("generics without defaults ok", {
  for (f in generics_w_no_default) {
    expect_true(exists(f, mode = "function"), info = f)
    expect_error(get(f, mode = "function")(2), "no applicable method", info = f)
    expect_output(print(methods(f)), "no methods found", info = f)
  }
})

---
title: "Guidelines for Developers of R Packages Interfacing with Stan"
author: "Stan Development Team"
date: "Latest release: `r format(Sys.Date(), format = '%B %Y')` (First version: November 2016)"
output: 
  rmarkdown::html_vignette:
    toc: true
vignette: >
  %\VignetteIndexEntry{Guidelines for Developers of R Packages Interfacing with Stan}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

## Note to developers

One of the coolest things about working on a project like Stan has been seeing
some of our users begin to develop tools for making Stan more accessible to
audiences that may otherwise not benefit from what Stan offers. In particular,
recently we have started seeing a growing number of R packages that provide
high-level interfaces to Stan, using the [__rstan__](https://mc-stan.org/rstan/) 
package for estimating models without requiring that the user be familiar with 
the Stan modeling language itself.

This is a great development and we would like to support such efforts going
forward, but to-date we have made little effort to coordinate the development of
these packages. To avoid a Wild West, so to speak, of Stan-based R packages, we
think it is important that developers make every effort to adhere to certain
guidelines in order to ensure these packages are of the highest possible quality
and provide the best possible experience for users. To that end, in this post we
present a set of recommendations for the development of R packages that
interface with Stan. These recommendations are based on software design
principles we value as well as many things we are learning as we continue
developing our own packages and review packages being developed by others. There
are exceptions to some of these recommendations (e.g., the __brms__ package is a
sensible exception to one of guidelines about Stan code), but we strongly
recommend trying to follow them whenever possible.

These recommendations are not set in stone. We expect them to evolve and we very
much appreciate feedback on how they can be improved. And, of course, we look
forward to seeing the packages you develop using Stan, so please let us know
about them!


## Guidelines for R packages providing interfaces to Stan

### General package structure and development

* The __rstantools__ package provides the `rstan_create_package()` function,
which you should use to create the basic structure of your package. (As of
`v2.0.0` this replaces the `rstan_package_skeleton` function.) This will set up
a package with functionality for pre-compiled Stan programs, in the style of the
[__rstanarm__](https://mc-stan.org/rstanarm/) package (source code:
https://github.com/stan-dev/rstanarm).

* Use version control (e.g., git).

* Unless you are developing proprietary private software, organize your code in 
a repository that is _public_ on [GitHub](https://github.com/) (or a similar
service, but preferably GitHub). It should be public even at early stages of
development, not only when officially released.  We recommend you add a note to
your README file on how to install the development version of your package, like
in the [__bayesplot__ README](https://github.com/stan-dev/bayesplot#installation)

* Unit testing is essential. There are several R packages that make it
relatively easy to write tests for your package. Most of our R packages (e.g.,
__rstanarm__, __brms__, __bayesplot__, __shinystan__, __loo__ and others) use
the [__testthat__](https://github.com/r-lib/testthat) package for this purpose,
but if you prefer a different testing framework that's fine. The
[__covr__](https://github.com/r-lib/covr) package is useful for calculating the
line coverage of your tests, and we recommend reaching a high level of coverage
before releasing a package. Good line coverage does not guarantee high quality
tests, but it's a good first step. We also recommend setting up automatic
testing of your package using GitHub Actions. See
https://github.com/r-lib/actions for useful templates.

### Stan code

* All Stan code for estimating models should be included in pre-written static
`.stan` files that are compiled when the package is built (see the Stan programs
directory in the __rstanarm__ repo for examples). You can also use
subdirectories to include code chunks to be used in multiple `.stan` files (again
see the __rstanarm__ repo for examples). If you set up your package using
`rstan_create_package` this structure will be created for you. This means that
__your package should NOT write a Stan program when the user calls a model
fitting function in your package__, but rather use only Stan programs you have
written by hand in advance (if you are working on a model for which you don't
think this is possible please let us know). There are several reasons for this.

* Pre-compiled Stan programs can be run by users of Windows or Mac OSX without
  having to install a C++ compiler, which dramatically expands the universe of
  potential users for your package.
* Pre-compiled Stan programs will run immediately when called, avoiding compilation time. 
* CRAN policy permits long installation times but imposes restrictions on the
  time consumed by examples and unit tests that are much shorter than the time 
  that it takes to compile even a simple Stan program. Thus, it is only possible
  to adequately test your package if it has pre-compiled Stan programs.
* Pre-compiled Stan programs can use custom C++ functions.  

To provide flexibility to users, your Stan programs can include branching logic 
(conditional statements) so that even with a small number of .stan files you can still 
allow for many different specifications to made by the user (see the .stan files in
__rstanarm__ for examples).

* Use best practices for Stan code. If the models you intend to implement are
discussed in the Stan manual or on the Stan users forum then you should follow
any recommendations that apply to your case. If you are unsure whether your Stan
programs can be made more efficient or more numerically stable then please ask
us on the Stan users forum. Especially ask us if you are unsure whether your
Stan programs are indeed estimating the intended model.

* Relatedly, prioritize safety over speed in your Stan code and sampler 
settings. For example, if you can write a program that runs faster but is 
potentially less stable, then at a minimum you should make the more stable 
version the default. This also means that, with rare exceptions, you should not 
change our recommended MCMC defaults (e.g. 4 chains, 1000+1000 iterations, NUTS 
not static HMC), unless you are setting the defaults to something more 
conservative. __rstanarm__ even goes one step further, making the default value
of the `adapt_delta` tuning parameter at least 0.95 for all models (rather than 
__rstan__'s default of 0.8) in order to reduce the step size and therefore also 
limit the potential for divergences. This means that __rstanarm__ models may
often run a bit slower than they need to if the user doesn't change the
defaults, but it also means that users face fewer situations in which they need
to know how to change the defaults and what the implications of changing the
defaults really are.

### R code and documentation

* Functions/methods that provide useful post-estimation functionality should be given 
the same names as the corresponding functions in __rstanarm__ (if applicable). 
For example, `posterior_predict()` to draw from the posterior predictive 
distribution, `posterior_interval()` for posterior uncertainty intervals, etc. To 
make this easier, these and similar __rstanarm__ functions have been converted 
to S3 methods for the stanreg objects created by __rstanarm__ and the S3 generic
functions are included here in the __rstantools__ package. Your package should
import the generics from __rstantools__ for whichever functions you want to
include in your package and then provide methods for the fitted model objects
returned by your model-fitting functions. For some other functions (e.g.
`as.matrix`) the generics are already available in base R or core R packages. To
be clear, we are not saying that the naming conventions used in
__rstanarm__/__rstantools__ are necessarily optimal. (If you think that one of 
our function names should be changed please let us know and suggest an 
alternative. If it is a substantial improvement we may consider renaming the 
function and deprecating the current version.) Rather, this guideline is 
intended to make function names consistent across Stan-based R packages, which 
will improve the user experience for those users who want to take advantage of a
variety of these packages. It will be a mess if every R package using Stan has 
different names for the same functionality.

* The [__bayesplot__](https://mc-stan.org/bayesplot/) package serves as the
back-end for plotting for __rstanarm__ (see for example `pp_check.stanreg` and
`plot.stanreg`), __brms__, and other packages, and we hope developers of other
Stan-based R packages will also use it. You can see all the other R packages
using __bayesplot__ in the _Reverse dependencies_ section of the __bayesplot__
[CRAN page](https://CRAN.R-project.org/package=bayesplot). For any plot you
intend to include in your package, if it is already available in __bayesplot__
then we recommend using the available version or suggesting (or contributing) a
better version. If it is not already available then there is a good chance we
will be interested in including it in __bayesplot__ if the plot would also be
useful for other developers.

* The [__posterior__](https://mc-stan.org/posterior/) package (new in 2021)
provides state of the art posterior inference diagnostics, various summaries of
draws in convenient formats, and functionality for converting between (and
manipulating) many different useful formats of draws from posterior or prior
distributions. We recommend using this functionality in your package or
recommending it to your users.

* Take documentation seriously. The documentation won't be perfect (we
constantly find holes in the doc for the R packages in the Stan ecosystem), but
you should make every effort to provide clear and thorough documentation. 


### Recommended resources

* Hadley Wickham and Jenny Bryan's [book on R packages](https://r-pkgs.org/). If you 
are interested in developing an R package that interfaces with Stan but are not an 
experienced package developer, we recommend this book, which is free to read
online. Even if you are an experienced developer of R packages, the book is
still a great resource.

* If you need help setting up your package or have questions about these 
guidelines the best places to go are the [Stan
Forums](https://discourse.mc-stan.org) and the [GitHub issue 
tracker](https://github.com/stan-dev/rstantools/issues) for the __rstantools__ 
package.

---
title: "Step by step guide for creating a package that depends on RStan"
author: "Stefan Siegert, Jonah Gabry, Martin Lysy, and Ben Goodrich"
date: "`r Sys.Date()`"
output:
  rmarkdown::html_vignette:
    toc: true
params:
  EVAL: !r identical(Sys.getenv("NOT_CRAN"), "true")
vignette: >
  %\VignetteIndexEntry{Step by step guide for creating a package that depends on RStan}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r SETTINGS-knitr, include=FALSE}
stopifnot(require(knitr))
opts_chunk$set(
  comment=NA,
  eval = if (isTRUE(exists("params"))) params$EVAL else FALSE
)
td <- tempdir()
PATH <- file.path(td, "rstanlm")
if(dir.exists(PATH)) {
  unlink(PATH, recursive = TRUE, force = TRUE)
}
```

## Introduction

In this vignette we will walk through the steps necessary for creating an
R package that depends on Stan by creating a package with one function
that fits a simple linear regression. Before continuing, we recommend that you first read the other vignette [_Guidelines for Developers of R Packages Interfacing with Stan_](https://mc-stan.org/rstantools/articles/developer-guidelines.html).

## Creating the package skeleton

The **rstantools** package offers two methods for adding Stan functionality
to R packages:

  * `rstan_create_package()`: set up a new R package with Stan programs
  * `use_rstan()`: add Stan functionality to an _existing_ R package

Here we will use `rstan_create_package()` to initialize a bare-bones package
directory. The name of our demo package will be __rstanlm__; it will fit a
simple linear regression model using Stan.

```{r rstan_create_package, eval=FALSE}
library("rstantools")
rstan_create_package(path = 'rstanlm')
```
```{r rstan_create_package-eval, echo=FALSE,warning=FALSE}
library("rstantools")
rstan_create_package(path = PATH, rstudio=FALSE, open=FALSE)
```

If we had existing `.stan` files to include with the package we could use the
optional `stan_files` argument to `rstan_create_package()` to include them.
Another option, which we'll use below, is to add the Stan files once the
basic structure of the package is in place.

We can now set the new working directory to the new package directory and view
the contents. (Note: if using RStudio then by default the newly created project
for the package will be opened in a new session and you will not need the call
to `setwd()`.)


```{r, eval=FALSE}
setwd("rstanlm")
list.files(all.files = TRUE)
```
```{r, echo=FALSE}
list.files(PATH, all.files = TRUE)
```

```{r, eval=FALSE}
file.show("DESCRIPTION")
```
```{r, echo=FALSE}
DES <- readLines(file.path(PATH, "DESCRIPTION"))
cat(DES, sep = "\n")
```

Some of the sections in the `DESCRIPTION` file need to be edited by hand (e.g.,
`Title`, `Author`, `Maintainer`, and `Description`, but these also can be set
with the `fields` argument to `rstan_create_package()`).  However,
`rstan_create_package()` has added the necessary packages and versions to
`Depends`, `Imports`, and `LinkingTo` to enable Stan functionality.

## Read-and-delete-me file

Before deleting the `Read-and-delete-me` file in the new package directory make
sure to read it because it contains some important instructions about
customizing your package:

```{r, eval=FALSE}
file.show("Read-and-delete-me")
```
```{r, echo=FALSE}
cat(readLines(file.path(PATH, "Read-and-delete-me")), sep = "\n")
```

You can move this file out of the directory, delete it, or list it in the
`.Rbuildignore` file if you want to keep it in the directory.

```{r, eval=FALSE}
file.remove('Read-and-delete-me')
```
```{r, echo=FALSE}
file.remove(file.path(PATH, 'Read-and-delete-me'))
```


## Stan files

Our package will call **rstan**'s `sampling()` method to use MCMC to fit a simple
linear regression model for an outcome variable `y` with a single predictor `x`.
After writing the necessary Stan program, the file should be saved with a
`.stan` extension in the `inst/stan` subdirectory. We'll save the
following program to `inst/stan/lm.stan`:

```{stan, output.var = "foo", eval = FALSE}
// Save this file as inst/stan/lm.stan
data {
  int<lower=1> N;
  vector[N] x;
  vector[N] y;
}
parameters {
  real intercept;
  real beta;
  real<lower=0> sigma;
}
model {
  // ... priors, etc.

  y ~ normal(intercept + beta * x, sigma);
}

```

```{r, include=FALSE}
stan_prog <- "
data {
  int<lower=1> N;
  vector[N] x;
  vector[N] y;
}
parameters {
  real intercept;
  real beta;
  real<lower=0> sigma;
}
model {
  // ... priors, etc.

  y ~ normal(intercept + beta * x, sigma);
}
"
writeLines(stan_prog, con = file.path(PATH, "inst", "stan", "lm.stan"))
rstan_config(PATH)
```

The `inst/stan` subdirectory can contain additional Stan programs if
required by your package. During installation, all Stan programs will be
compiled and saved in the list `stanmodels` that can then be used by R function
in the package. The rule is that the Stan program compiled from the model code
in `inst/stan/foo.stan` is stored as list element `stanmodels$foo`. Thus, the
filename of the Stan program in the `inst/stan` directory should not contain
spaces or dashes and nor should it start with a number or utilize non-ASCII
characters.

## R files

We next create the file `R/lm_stan.R` where we define the function `lm_stan()`
in which our compiled Stan model is being used.  Setting the
`rstan_create_package()` argument `roxygen = TRUE` (the default value) enables
[__roxygen2__](https://CRAN.R-project.org/package=roxygen2) documentation for
the package functions.  The following comment block placed in `lm_stan.R`
ensures that the function has a help file and that it is added to the package
`NAMESPACE`:

```{r}
# Save this file as `R/lm_stan.R`

#' Bayesian linear regression with Stan
#'
#' @export
#' @param x Numeric vector of input values.
#' @param y Numeric vector of output values.
#' @param ... Arguments passed to `rstan::sampling` (e.g. iter, chains).
#' @return An object of class `stanfit` returned by `rstan::sampling`
#'
lm_stan <- function(x, y, ...) {
  standata <- list(x = x, y = y, N = length(y))
  out <- rstan::sampling(stanmodels$lm, data = standata, ...)
  return(out)
}

```


```{r, include=FALSE}
Rcode <- "
#' Bayesian linear regression with Stan
#'
#' @export
#' @param x Numeric vector of input values.
#' @param y Numeric vector of output values.
#' @param ... Arguments passed to `rstan::sampling`.
#' @return An object of class `stanfit` returned by `rstan::sampling`
lm_stan <- function(x, y, ...) {
  out <- rstan::sampling(stanmodels$lm, data=list(x=x, y=y, N=length(y)), ...)
  return(out)
}
"
writeLines(Rcode, con = file.path(PATH, "R", "lm_stan.R"))
```

When __roxygen2__ documentation is enabled, a top-level package file
`R/rstanlm-package.R` is created by `rstan_create_package()` to import necessary
functions for other packages and to set up the package for compiling Stan C++
code:

```{r, eval=FALSE}
file.show(file.path("R", "rstanlm-package.R"))
```
```{r, echo=FALSE}
cat(readLines(file.path(PATH, "R", "rstanlm-package.R")), sep = "\n")
```

The `#' @description` section can be manually edited to provided specific
information about the package.

## Documentation

With __roxygen__ documentation enabled, we need to generate the documentation
for `lm_stan` and update the `NAMESPACE` so the function is exported, i.e.,
available to users when the package is installed. This can be done with the
function `roxygen2::roxygenize()`, which needs to be called twice initially.

```{r, eval = FALSE}
try(roxygen2::roxygenize(load_code = rstantools_load_code), silent = TRUE)
roxygen2::roxygenize()
```

```{r, echo=FALSE, results="hide"}
try(roxygen2::roxygenize(PATH, load_code = rstantools_load_code), silent = TRUE)
roxygen2::roxygenize(PATH)
```

## Install and use

Finally, the package is ready to be installed:

```{r,eval=FALSE}
# using ../rstanlm because already inside the rstanlm directory
install.packages("../rstanlm", repos = NULL, type = "source")
```
```{r,echo=FALSE}
install.packages(PATH, repos = NULL, type = "source")
```

It is also possible to use `devtools::install(quick=FALSE)` to install the
package. The argument `quick=FALSE` is necessary if you want to recompile the
Stan models. Going forward, if you only make a  change to the R code or the
documentation, you can set `quick=TRUE` to speed up the process, or use
`devtools::load_all()`.

After installation, the package can be loaded and used like any other R package:

```{r, eval=FALSE}
library("rstanlm")
```

```{r}
fit <- lm_stan(y = rnorm(10), x = rnorm(10), 
               # arguments passed to sampling
               iter = 2000, refresh = 500)
print(fit)
```

```{r, echo=FALSE}
unlink(PATH, recursive = TRUE, force = TRUE)
```

## Advanced options

Details can be found in the documentation for `rstan_create_package()` so we
only mention some of these briefly here:

* Running `rstan_create_package()` with `auto_config = TRUE` (the default value)
automatically synchronizes the Stan C++ files with the `.stan` model files
located in `inst/stan`, although this creates a dependency of your package on
__rstantools__ itself (i.e., __rstantools__ must be installed for your package
to work).  Setting `auto_config = FALSE` removes this dependency, at the cost of
having to manually synchronize Stan C++ files by running `rstan_config()` every
time a package `.stan` file is added, removed, or even just modified.

* The function `use_rstan()` can be used to add Stan functionality to an
existing package, instead of building the package from scratch.
  * Note: If you are already using roxygen in your package, you'll have to use
    roxygen to update your Namespace file via the `R/<package-name>-package.R` file.
    Check the roxygen documentation for more details.

## Adding additional Stan models to an existing R package with Stan models

One may add additional Stan models to an existing package.
The following steps are required if one is using `devtools`:

1.  Add new Stan file, e.g., `inst/stan/new.stan`
2.  Run `pkgbuild::compile_dll()` to preform a fake R CMD install.
3.  Run  `roxygen2::roxygenize()` to update the documentation.
4.  Run ` devtools::install()` to install the package locally.

## Links

* [_Guidelines for Developers of R Packages Interfacing with Stan_](https://mc-stan.org/rstantools/articles/developer-guidelines.html)

* Ask a question at the [Stan Forums](https://discourse.mc-stan.org/)

* [R packages](https://r-pkgs.org/) by Hadley Wickham and Jenny Bryan provides
a solid foundation in R package development as well as the release process.

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/predictive_interval.R
\name{predictive_interval}
\alias{predictive_interval}
\alias{predictive_interval.default}
\title{Generic function for predictive intervals}
\usage{
predictive_interval(object, ...)

\method{predictive_interval}{default}(object, prob = 0.9, ...)
}
\arguments{
\item{object}{The object to use.}

\item{...}{Arguments passed to methods. See the methods in the \pkg{rstanarm}
package for examples.}

\item{prob}{A number \eqn{p \in (0,1)}{p (0 < p < 1)} indicating the desired
probability mass to include in the intervals.}
}
\value{
\code{predictive_interval()} methods should return a matrix with two
columns and as many rows as data points being predicted. For a given value
of \code{prob}, \eqn{p}, the columns correspond to the lower and upper
\eqn{100p}\\% interval limits and have the names \eqn{100\alpha/2}\\% and
\eqn{100(1 - \alpha/2)}\\%, where \eqn{\alpha = 1-p}. For example, if
\code{prob=0.9} is specified (a \eqn{90}\\% interval), then the column names
would be \code{"5\%"} and \code{"95\%"}, respectively.

The default method just takes \code{object} to be a matrix and computes
quantiles, with \code{prob} defaulting to \code{0.9}.
}
\description{
See \href{https://mc-stan.org/rstanarm/reference/predictive_interval.stanreg.html}{predictive_interval.stanreg()}
in the \pkg{rstanarm} package for an example.
}
\examples{
# Default method takes a numeric matrix (of draws from posterior
# predictive distribution)
ytilde <- matrix(rnorm(100 * 5, sd = 2), 100, 5) # fake draws
predictive_interval(ytilde, prob = 0.8)

# Also see help("predictive_interval", package = "rstanarm")

}
\seealso{
\itemize{
\item The \pkg{rstanarm} package (\href{https://mc-stan.org/rstanarm/}{mc-stan.org/rstanarm})
for example methods
(\href{https://CRAN.R-project.org/package=rstanarm}{CRAN},
\href{https://github.com/stan-dev/rstanarm}{GitHub}).
}

\itemize{
\item Guidelines and recommendations for developers of \R packages
interfacing with Stan and a demonstration getting a simple package working
can be found in the vignettes included with \pkg{rstantools} and at
\href{https://mc-stan.org/rstantools/articles/}{mc-stan.org/rstantools/articles}.
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/prior_summary.R
\name{prior_summary}
\alias{prior_summary}
\alias{prior_summary.default}
\title{Generic function for extracting information about prior distributions}
\usage{
prior_summary(object, ...)

\method{prior_summary}{default}(object, ...)
}
\arguments{
\item{object}{The object to use.}

\item{...}{Arguments passed to methods. See the methods in the \pkg{rstanarm}
package for examples.}
}
\value{
\code{prior_summary()} methods should return an object containing
information about the prior distribution(s) used for the given model.
The structure of this object will depend on the method.

The default method just returns \code{object$prior.info}, which is
\code{NULL} if there is no \code{'prior.info'} element.
}
\description{
See \href{https://mc-stan.org/rstanarm/reference/prior_summary.stanreg.html}{prior_summary.stanreg()}
in the \pkg{rstanarm} package for an example.
}
\examples{
# See help("prior_summary", package = "rstanarm")

}
\seealso{
\itemize{
\item The \pkg{rstanarm} package (\href{https://mc-stan.org/rstanarm/}{mc-stan.org/rstanarm})
for example methods
(\href{https://CRAN.R-project.org/package=rstanarm}{CRAN},
\href{https://github.com/stan-dev/rstanarm}{GitHub}).
}

\itemize{
\item Guidelines and recommendations for developers of \R packages
interfacing with Stan and a demonstration getting a simple package working
can be found in the vignettes included with \pkg{rstantools} and at
\href{https://mc-stan.org/rstantools/articles/}{mc-stan.org/rstantools/articles}.
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/predictive_error.R
\name{predictive_error}
\alias{predictive_error}
\alias{predictive_error.default}
\title{Generic function and default method for predictive errors}
\usage{
predictive_error(object, ...)

\method{predictive_error}{default}(object, y, ...)
}
\arguments{
\item{object}{The object to use.}

\item{...}{Arguments passed to methods. See the methods in the \pkg{rstanarm}
package for examples.}

\item{y}{For the default method, a vector of \code{y} values the same length as
the number of columns in the matrix used as \code{object}.}
}
\value{
\code{predictive_error()} methods should return a \eqn{D} by \eqn{N}
matrix, where \eqn{D} is the number of draws from the posterior predictive
distribution and \eqn{N} is the number of data points being predicted per
draw.

The default method just takes \code{object} to be a matrix and \code{y} to be a
vector.
}
\description{
Generic function and default method for computing predictive errors
\eqn{y - y^{rep}}{y - yrep} (in-sample, for observed \eqn{y}) or
\eqn{y - \tilde{y}}{y - ytilde} (out-of-sample, for new or held-out \eqn{y}).
See \href{https://mc-stan.org/rstanarm/reference/predictive_error.stanreg.html}{predictive_error.stanreg()}
in the \pkg{rstanarm} package for an example.
}
\examples{
# default method
y <- rnorm(10)
ypred <- matrix(rnorm(500), 50, 10)
pred_errors <- predictive_error(ypred, y)
dim(pred_errors)
head(pred_errors)

# Also see help("predictive_error", package = "rstanarm")

}
\seealso{
\itemize{
\item The \pkg{rstanarm} package (\href{https://mc-stan.org/rstanarm/}{mc-stan.org/rstanarm})
for example methods
(\href{https://CRAN.R-project.org/package=rstanarm}{CRAN},
\href{https://github.com/stan-dev/rstanarm}{GitHub}).
}

\itemize{
\item Guidelines and recommendations for developers of \R packages
interfacing with Stan and a demonstration getting a simple package working
can be found in the vignettes included with \pkg{rstantools} and at
\href{https://mc-stan.org/rstantools/articles/}{mc-stan.org/rstantools/articles}.
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/init_cpp.R
\name{init_cpp}
\alias{init_cpp}
\title{Register functions implemented in C++}
\usage{
init_cpp(name, path)
}
\arguments{
\item{name}{The name of your package as a string.}

\item{path}{The path to the root directory for your package as a string. If
not specified it is assumed that this is already the current working
directory.}
}
\value{
This function is only called for its side effect of writing the
necessary \code{init.cpp} file to the package's \verb{src/} directory.
}
\description{
If you set up your package using \code{rstan_package_skeleton()} before
version \verb{1.2.1} of \pkg{rstantools} it may be necessary for you to call
this function yourself in order to pass \verb{R CMD check} in \R
\verb{>= 3.4}. If you used \code{rstan_package_skeleton()} in \pkg{rstantools} version
\verb{1.2.1} or later then this has already been done automatically.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/nsamples.R
\name{nsamples}
\alias{nsamples}
\title{Generic function for extracting the number of posterior samples}
\usage{
nsamples(object, ...)
}
\arguments{
\item{object}{The object to use.}

\item{...}{Arguments passed to methods. See the methods in the \pkg{rstanarm}
package for examples.}
}
\description{
Extract the number of posterior samples stored in a fitted Bayesian model.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/rstantools-package.R
\docType{package}
\name{rstantools-package}
\alias{rstantools-package}
\alias{rstantools}
\title{Tools for Developing \R Packages Interfacing with Stan}
\description{
\if{html}{
  \figure{logo.svg}{options: width="50" alt="mc-stan.org"}
} \emph{Stan Development Team}

The \pkg{rstantools} package provides various tools for developers of \R
packages interfacing with Stan (\url{https://mc-stan.org}), including
functions to set up the required package structure, S3 generic methods to
unify function naming across Stan-based \R packages, and vignettes with
guidelines for developers. To get started building a package see
\code{\link[=rstan_create_package]{rstan_create_package()}}.
}
\seealso{
\itemize{
\item Guidelines and recommendations for developers of \R packages
interfacing with Stan and a demonstration getting a simple package working
can be found in the vignettes included with \pkg{rstantools} and at
\href{https://mc-stan.org/rstantools/articles/}{mc-stan.org/rstantools/articles}.
}

\itemize{
\item After reading the guidelines for developers, if you have trouble setting up
your package let us know on the the \href{https://discourse.mc-stan.org}{Stan Forums}
or at the \pkg{rstantools} GitHub \href{https://github.com/stan-dev/rstantools/issues}{issue tracker}.
}

Useful links:
\itemize{
  \item \url{https://mc-stan.org/rstantools/}
  \item \url{https://discourse.mc-stan.org/}
  \item Report bugs at \url{https://github.com/stan-dev/rstantools/issues}
}

}
\author{
\strong{Maintainer}: Jonah Gabry \email{jgabry@gmail.com}

Authors:
\itemize{
  \item Ben Goodrich \email{benjamin.goodrich@columbia.edu}
  \item Martin Lysy \email{mlysy@uwaterloo.ca}
  \item Andrew Johnson
}

Other contributors:
\itemize{
  \item Hamada S. Badr [contributor]
  \item Marco Colombo [contributor]
  \item Stefan Siegert [contributor]
  \item Trustees of Columbia University [copyright holder]
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/rstan_create_package.R
\name{rstan_create_package}
\alias{rstan_create_package}
\alias{rstan_package_skeleton}
\title{Create a new \R package with compiled Stan programs}
\usage{
rstan_create_package(
  path,
  fields = NULL,
  rstudio = TRUE,
  open = TRUE,
  stan_files = character(),
  roxygen = TRUE,
  travis = FALSE,
  license = TRUE,
  auto_config = TRUE
)
}
\arguments{
\item{path}{The path to the new package to be created (terminating in the
package name).}

\item{fields, rstudio, open}{Same as \code{\link[usethis:create_package]{usethis::create_package()}}. See
the documentation for that function, especially the note in the
\strong{Description} section about the side effect of changing the active
project.}

\item{stan_files}{A character vector with paths to \code{.stan} files to include
in the package.}

\item{roxygen}{Should \pkg{roxygen2} be used for documentation?  Defaults to
\code{TRUE}. If so, a file `R/{pkgname}-package.R`` is added to the package with
roxygen tags for the required import lines. See the \strong{Note} section below
for advice specific to the latest versions of \pkg{roxygen2}.}

\item{travis}{Should a \code{.travis.yml} file be added to the package directory?
This argument is now deprecated. We recommend using GitHub Actions to set
up automated testings for your package. See
https://github.com/r-lib/actions for useful templates.}

\item{license}{Logical or character; whether or not to paste the contents of
a \code{license.stan} file at the top of all Stan code, or path to such a
file.  If \code{TRUE} (the default) adds the \verb{GPL (>= 3)} license (see
\strong{Details}).}

\item{auto_config}{Whether to automatically configure Stan functionality
whenever the package gets installed (see \strong{Details}). Defaults to \code{TRUE}.}
}
\description{
\if{html}{\figure{stanlogo.png}{options: width="25" alt="https://mc-stan.org/about/logo/"}}
The \code{rstan_create_package()} function helps get you started developing a
new \R package that interfaces with Stan via the \pkg{rstan} package. First
the basic package structure is set up via \code{\link[usethis:create_package]{usethis::create_package()}}.
Then several adjustments are made so the package can include Stan programs
that can be built into binary versions (i.e., pre-compiled Stan C++ code).

The \strong{Details} section below describes the process and the
\strong{See Also} section provides links to recommendations for developers
and a step-by-step walk-through.

As of version \verb{2.0.0} of \pkg{rstantools} the
\code{rstan_package_skeleton()} function is defunct and only
\code{rstan_create_package()} is supported.
}
\details{
This function first creates a regular \R package using
\code{usethis::create_package()}, then adds the infrastructure required to compile
and export \code{stanmodel} objects. In the package root directory, the user's
Stan source code is located in:
\preformatted{
inst/
  |_stan/
  |   |_include/
  |
  |_include/
}
All \code{.stan} files containing instructions to build a \code{stanmodel}
object must be placed in \code{inst/stan}.  Other \code{.stan} files go in
any \verb{stan/} subdirectory, to be invoked by Stan's \verb{#include}
mechanism, e.g.,
\preformatted{
#include "include/mylib.stan"
#include "data/preprocess.stan"
}
See \pkg{rstanarm} for many examples.

The folder \code{inst/include} is for all user C++ files associated with the
Stan programs.  In this folder, the only file to directly interact with the
Stan C++ library is \code{stan_meta_header.hpp}; all other \verb{#include}
directives must be channeled through here.

The final step of the package creation is to invoke
\code{\link[=rstan_config]{rstan_config()}}, which creates the following files for
interfacing with Stan objects from \R:
\itemize{
\item \code{src} contains the \verb{stan_ModelName\{.cc/.hpp\}} pairs
associated with all \code{ModelName.stan} files in \code{inst/stan} which
define \code{stanmodel} objects.
\item \code{src/Makevars[.win]} which link to the \code{StanHeaders} and
Boost (\code{BH}) libraries.
\item \code{R/stanmodels.R} loads the C++ modules containing the
\code{stanmodel} class definitions, and assigns an \R instance of each
\code{stanmodel} object to a \code{stanmodels} list (with names
corresponding to the names of the Stan files).
}

When \code{auto_config = TRUE}, a \code{configure[.win]} file is added to the
package, calling \code{\link[=rstan_config]{rstan_config()}} whenever the package is installed.
Consequently, the package must list \pkg{rstantools} in the \code{DESCRIPTION}
Imports field for this mechanism to work.  Setting \code{auto_config = FALSE}
removes the package's dependency on \pkg{rstantools}, but the package then
must be manually configured by running \code{\link[=rstan_config]{rstan_config()}} whenever
\code{stanmodel} files in \code{inst/stan} are added, removed, or modified.

In order to enable Stan functionality, \pkg{\link{rstantools}}
copies some files to your package.  Since these files are licensed as GPL= 3, the same license applies to your package should you choose todistribute it.  Even if you don't use \pkg{\link{rstantools}} to createyour package, it is likely that you will be linking to \pkg{\link{Rcpp}} toexport the Stan C++ stanmodel objects to \R.  Since\pkg{\link{Rcpp}} is released under GPL >= 2, the same license would applyto your package upon distribution.

Authors willing to license their Stan programs of general interest
under the GPL are invited to contribute their \code{.stan} files and
supporting \R code to the \pkg{rstanarm} package.
}
\note{
For \pkg{devtools} users, because of changes in the latest versions of
\pkg{roxygen2} it may be necessary to run \code{pkgbuild::compile_dll()}
once before \code{devtools::document()} will work.
}
\section{Using the pre-compiled Stan programs in your package}{
 The
\code{stanmodel} objects corresponding to the Stan programs included with your
package are stored in a list called \code{stanmodels}. To run one of the Stan
programs from within an R function in your package just pass the
appropriate element of the \code{stanmodels} list to one of the \pkg{rstan}
functions for model fitting (e.g., \code{sampling()}). For example, for a Stan
program \code{"foo.stan"} you would use \code{rstan::sampling(stanmodels$foo, ...)}.
}

\seealso{
\itemize{
\item \code{\link[=use_rstan]{use_rstan()}} for adding Stan functionality to an existing
\R package and \code{\link[=rstan_config]{rstan_config()}} for updating an existing package
when its Stan files are changed.
\item The \pkg{rstanarm} package \href{https://github.com/stan-dev/rstanarm}{repository}
on GitHub.
}

\itemize{
\item Guidelines and recommendations for developers of \R packages
interfacing with Stan and a demonstration getting a simple package working
can be found in the vignettes included with \pkg{rstantools} and at
\href{https://mc-stan.org/rstantools/articles/}{mc-stan.org/rstantools/articles}.
}

\itemize{
\item After reading the guidelines for developers, if you have trouble setting up
your package let us know on the the \href{https://discourse.mc-stan.org}{Stan Forums}
or at the \pkg{rstantools} GitHub \href{https://github.com/stan-dev/rstantools/issues}{issue tracker}.
}

\itemize{
\item The useR2016 presentation
\href{https://channel9.msdn.com/Events/useR-international-R-User-conferences/useR-International-R-User-2017-Conference/How-to-Use-RStan-to-Estimate-Models-in-External-R-Packages}{How to Use (R)Stan to Estimate Models in External R Packages.}
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/posterior_interval.R
\name{posterior_interval}
\alias{posterior_interval}
\alias{posterior_interval.default}
\title{Generic function and default method for posterior uncertainty intervals}
\usage{
posterior_interval(object, ...)

\method{posterior_interval}{default}(object, prob = 0.9, ...)
}
\arguments{
\item{object}{The object to use.}

\item{...}{Arguments passed to methods. See the methods in the \pkg{rstanarm}
package for examples.}

\item{prob}{A number \eqn{p \in (0,1)}{p (0 < p < 1)} indicating the desired
probability mass to include in the intervals.}
}
\value{
\code{posterior_interval()} methods should return a matrix with two
columns and as many rows as model parameters (or a subset of parameters
specified by the user). For a given value of \code{prob}, \eqn{p}, the
columns correspond to the lower and upper \eqn{100p}\\% interval limits and
have the names \eqn{100\alpha/2}\\% and \eqn{100(1 - \alpha/2)}\\%, where
\eqn{\alpha = 1-p}. For example, if \code{prob=0.9} is specified (a
\eqn{90}\\% interval), then the column names would be \code{"5\%"} and
\code{"95\%"}, respectively.

The default method just takes \code{object} to be a matrix (one column per
parameter) and computes quantiles, with \code{prob} defaulting to \code{0.9}.
}
\description{
These intervals are often referred to as credible intervals, but we use the
term uncertainty intervals to highlight the fact that wider intervals
correspond to greater uncertainty. See
\href{https://mc-stan.org/rstanarm/reference/posterior_interval.stanreg.html}{posterior_interval.stanreg()}
in the \pkg{rstanarm} package for an example.
}
\examples{
# Default method takes a numeric matrix (of posterior draws)
draws <- matrix(rnorm(100 * 5), 100, 5) # fake draws
colnames(draws) <- paste0("theta_", 1:5)
posterior_interval(draws)

# Also see help("posterior_interval", package = "rstanarm")

}
\seealso{
\itemize{
\item The \pkg{rstanarm} package (\href{https://mc-stan.org/rstanarm/}{mc-stan.org/rstanarm})
for example methods
(\href{https://CRAN.R-project.org/package=rstanarm}{CRAN},
\href{https://github.com/stan-dev/rstanarm}{GitHub}).
}

\itemize{
\item Guidelines and recommendations for developers of \R packages
interfacing with Stan and a demonstration getting a simple package working
can be found in the vignettes included with \pkg{rstantools} and at
\href{https://mc-stan.org/rstantools/articles/}{mc-stan.org/rstantools/articles}.
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/rstan_package_utils.R
\name{rstantools_load_code}
\alias{rstantools_load_code}
\title{Helper function for loading code in roxygenise}
\usage{
rstantools_load_code(path, trace = TRUE, ...)
}
\arguments{
\item{path}{Path to directory containing code to load}

\item{trace}{Whether to print file names as they are loaded}

\item{...}{Additional arguments passed to \code{\link{source}}}
}
\value{
\code{NULL}
}
\description{
Adapted from the \code{sourceDir} function defined
by \code{example(source)}.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/posterior_linpred.R
\name{posterior_linpred}
\alias{posterior_linpred}
\title{Generic function for accessing the posterior distribution of the linear
predictor}
\usage{
posterior_linpred(object, transform = FALSE, ...)
}
\arguments{
\item{object}{The object to use.}

\item{transform}{Should the linear predictor be transformed using the
inverse-link function? The default is \code{FALSE}, in which case the
untransformed linear predictor is returned.}

\item{...}{Arguments passed to methods. See the methods in the \pkg{rstanarm}
package for examples.}
}
\value{
\code{posterior_linpred()} methods should return a \eqn{D} by \eqn{N}
matrix, where \eqn{D} is the number of draws from the posterior
distribution distribution and \eqn{N} is the number of data points.
}
\description{
Extract the posterior draws of the linear predictor, possibly transformed by
the inverse-link function. See
\href{https://mc-stan.org/rstanarm/reference/posterior_linpred.stanreg.html}{posterior_linpred.stanreg()}
in the \pkg{rstanarm} package for an example.
}
\examples{
# See help("posterior_linpred", package = "rstanarm")

}
\seealso{
\itemize{
\item The \pkg{rstanarm} package (\href{https://mc-stan.org/rstanarm/}{mc-stan.org/rstanarm})
for example methods
(\href{https://CRAN.R-project.org/package=rstanarm}{CRAN},
\href{https://github.com/stan-dev/rstanarm}{GitHub}).
}

\itemize{
\item Guidelines and recommendations for developers of \R packages
interfacing with Stan and a demonstration getting a simple package working
can be found in the vignettes included with \pkg{rstantools} and at
\href{https://mc-stan.org/rstantools/articles/}{mc-stan.org/rstantools/articles}.
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/log_lik.R
\name{log_lik}
\alias{log_lik}
\title{Generic function for pointwise log-likelihood}
\usage{
log_lik(object, ...)
}
\arguments{
\item{object}{The object to use.}

\item{...}{Arguments passed to methods. See the methods in the \pkg{rstanarm}
package for examples.}
}
\value{
\code{log_lik()} methods should return a \eqn{S} by \eqn{N} matrix,
where \eqn{S} is the size of the posterior sample (the number of draws from
the posterior distribution) and \eqn{N} is the number of data points.
}
\description{
We define a new function \code{log_lik()} rather than a
\code{\link[stats:logLik]{stats::logLik()}} method because (in addition to the conceptual
difference) the documentation for \code{logLik()} states that the return value
will be a single number, whereas \code{log_lik()} returns a matrix. See
the \href{https://mc-stan.org/rstanarm/reference/log_lik.stanreg.html}{log_lik.stanreg()}
method in the \pkg{rstanarm} package for an example of defining a method.
}
\examples{
# See help("log_lik", package = "rstanarm")

}
\seealso{
\itemize{
\item The \pkg{rstanarm} package (\href{https://mc-stan.org/rstanarm/}{mc-stan.org/rstanarm})
for example methods
(\href{https://CRAN.R-project.org/package=rstanarm}{CRAN},
\href{https://github.com/stan-dev/rstanarm}{GitHub}).
}

\itemize{
\item Guidelines and recommendations for developers of \R packages
interfacing with Stan and a demonstration getting a simple package working
can be found in the vignettes included with \pkg{rstantools} and at
\href{https://mc-stan.org/rstantools/articles/}{mc-stan.org/rstantools/articles}.
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/posterior_predict.R
\name{posterior_predict}
\alias{posterior_predict}
\title{Generic function for drawing from the posterior predictive distribution}
\usage{
posterior_predict(object, ...)
}
\arguments{
\item{object}{The object to use.}

\item{...}{Arguments passed to methods. See the methods in the \pkg{rstanarm}
package for examples.}
}
\value{
\code{posterior_predict()} methods should return a \eqn{D} by \eqn{N}
matrix, where \eqn{D} is the number of draws from the posterior predictive
distribution and \eqn{N} is the number of data points being predicted per
draw.
}
\description{
Draw from the posterior predictive distribution of the outcome. See
\href{https://mc-stan.org/rstanarm/reference/posterior_predict.stanreg.html}{posterior_predict.stanreg()}
in the \pkg{rstanarm} package for an example.
}
\examples{
# See help("posterior_predict", package = "rstanarm")

}
\seealso{
\itemize{
\item The \pkg{rstanarm} package (\href{https://mc-stan.org/rstanarm/}{mc-stan.org/rstanarm})
for example methods
(\href{https://CRAN.R-project.org/package=rstanarm}{CRAN},
\href{https://github.com/stan-dev/rstanarm}{GitHub}).
}

\itemize{
\item Guidelines and recommendations for developers of \R packages
interfacing with Stan and a demonstration getting a simple package working
can be found in the vignettes included with \pkg{rstantools} and at
\href{https://mc-stan.org/rstantools/articles/}{mc-stan.org/rstantools/articles}.
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/bayes_R2.R
\name{bayes_R2}
\alias{bayes_R2}
\alias{bayes_R2.default}
\alias{loo_R2}
\title{Generic function and default method for Bayesian R-squared}
\usage{
bayes_R2(object, ...)

\method{bayes_R2}{default}(object, y, ...)

loo_R2(object, ...)
}
\arguments{
\item{object}{The object to use.}

\item{...}{Arguments passed to methods. See the methods in the \pkg{rstanarm}
package for examples.}

\item{y}{For the default method, a vector of \code{y} values the same length
as the number of columns in the matrix used as \code{object}.}
}
\value{
\code{bayes_R2()} and \code{loo_R2()} methods should return a vector of
length equal to the posterior sample size.

The default \code{bayes_R2()} method just takes \code{object} to be a matrix of y-hat
values (one column per observation, one row per posterior draw) and \code{y} to
be a vector with length equal to \code{ncol(object)}.
}
\description{
Generic function and default method for Bayesian version of R-squared for
regression models. A generic for LOO-adjusted R-squared is also provided. See
the \href{https://mc-stan.org/rstanarm/reference/bayes_R2.stanreg.html}{bayes_R2.stanreg()}
method in the \pkg{rstanarm} package for an example of defining a method.
}
\references{
Andrew Gelman, Ben Goodrich, Jonah Gabry, and Aki Vehtari (2018). R-squared
for Bayesian regression models. \emph{The American Statistician}, to appear.
DOI: 10.1080/00031305.2018.1549100.
(\href{http://www.stat.columbia.edu/~gelman/research/published/bayes_R2_v3.pdf}{Preprint},
\href{https://avehtari.github.io/bayes_R2/bayes_R2.html}{Notebook})
}
\seealso{
\itemize{
\item The \pkg{rstanarm} package (\href{https://mc-stan.org/rstanarm/}{mc-stan.org/rstanarm})
for example methods
(\href{https://CRAN.R-project.org/package=rstanarm}{CRAN},
\href{https://github.com/stan-dev/rstanarm}{GitHub}).
}

\itemize{
\item Guidelines and recommendations for developers of \R packages
interfacing with Stan and a demonstration getting a simple package working
can be found in the vignettes included with \pkg{rstantools} and at
\href{https://mc-stan.org/rstantools/articles/}{mc-stan.org/rstantools/articles}.
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/loo-functions.R
\name{loo-prediction}
\alias{loo-prediction}
\alias{loo_linpred}
\alias{loo_epred}
\alias{loo_predict}
\alias{loo_predictive_interval}
\alias{loo_pit}
\alias{loo_pit.default}
\title{Generic functions for LOO predictions}
\usage{
loo_linpred(object, ...)

loo_epred(object, ...)

loo_predict(object, ...)

loo_predictive_interval(object, ...)

loo_pit(object, ...)

\method{loo_pit}{default}(object, y, lw, ...)
}
\arguments{
\item{object}{The object to use.}

\item{...}{Arguments passed to methods. See the methods in the \pkg{rstanarm}
package for examples.}

\item{y}{For the default method of \code{loo_pit()}, a vector of \code{y} values the
same length as the number of columns in the matrix used as \code{object}.}

\item{lw}{For the default method of \code{loo_pit()}, a matrix of log-weights of
the same length as the number of columns in the matrix used as \code{object}.}
}
\value{
\code{loo_predict()}, \code{loo_epred()}, \code{loo_linpred()}, and \code{loo_pit()}
(probability integral transform) methods should return a vector with
length equal to the number of observations in the data.
For discrete observations, probability integral transform is randomised to
ensure theoretical uniformity. Fix random seed for reproducible results
with discrete data. For more details, see Czado et al. (2009).
\code{loo_predictive_interval()} methods should return a two-column matrix
formatted in the same way as for \code{\link[=predictive_interval]{predictive_interval()}}.
}
\description{
See the methods in the \pkg{rstanarm} package for examples.
}
\references{
Czado, C., Gneiting, T., and Held, L. (2009).
Predictive Model Assessment for Count Data.
\emph{Biometrics}. 65(4), 1254-1261.
\doi{10.1111/j.1541-0420.2009.01191.x}.
}
\seealso{
\itemize{
\item The \pkg{rstanarm} package (\href{https://mc-stan.org/rstanarm/}{mc-stan.org/rstanarm})
for example methods
(\href{https://CRAN.R-project.org/package=rstanarm}{CRAN},
\href{https://github.com/stan-dev/rstanarm}{GitHub}).
}

\itemize{
\item Guidelines and recommendations for developers of \R packages
interfacing with Stan and a demonstration getting a simple package working
can be found in the vignettes included with \pkg{rstantools} and at
\href{https://mc-stan.org/rstantools/articles/}{mc-stan.org/rstantools/articles}.
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/use_rstan.R
\name{use_rstan}
\alias{use_rstan}
\title{Add Stan infrastructure to an existing package}
\usage{
use_rstan(pkgdir = ".", license = TRUE, auto_config = TRUE)
}
\arguments{
\item{pkgdir}{Path to package root folder.}

\item{license}{Logical or character; whether or not to paste the contents of
a \code{license.stan} file at the top of all Stan code, or path to such a
file.  If \code{TRUE} (the default) adds the \verb{GPL (>= 3)} license (see
\strong{Details}).}

\item{auto_config}{Whether to automatically configure Stan functionality
whenever the package gets installed (see \strong{Details}). Defaults to \code{TRUE}.}
}
\value{
Invisibly, \code{TRUE} or \code{FALSE} indicating whether or not any files or
folders where created or modified.
}
\description{
Add Stan infrastructure to an existing \R package. To create a \emph{new} package
containing Stan programs use \code{\link[=rstan_create_package]{rstan_create_package()}} instead.
}
\details{
Prepares a package to compile and use Stan code by performing the
following steps:
\enumerate{
\item Create \code{inst/stan} folder where all \code{.stan} files defining
Stan models should be stored.
\item Create \code{inst/stan/include} where optional \code{license.stan} file is stored.
\item Create \code{inst/include/stan_meta_header.hpp} to include optional header
files used by Stan code.
\item Create \code{src} folder (if it doesn't exist) to contain the Stan C++ code.
\item Create \code{R} folder (if it doesn't exist) to contain wrapper code to expose
Stan C++ classes to \R.
\item Update \code{DESCRIPTION} file to contain all needed dependencies to compile
Stan C++ code.
\item If \code{NAMESPACE} file is generic (i.e., created by \code{\link[=rstan_create_package]{rstan_create_package()}}),
append \code{import(Rcpp, methods)}, \code{importFrom(rstan, sampling)},
and \code{useDynLib} directives.  If \code{NAMESPACE} is not generic, display message
telling user what to add to \code{NAMESPACE} for themselves.
}

When \code{auto_config = TRUE}, a \code{configure[.win]} file is added to the
package, calling \code{\link[=rstan_config]{rstan_config()}} whenever the package is installed.
Consequently, the package must list \pkg{rstantools} in the \code{DESCRIPTION}
Imports field for this mechanism to work.  Setting \code{auto_config = FALSE}
removes the package's dependency on \pkg{rstantools}, but the package then
must be manually configured by running \code{\link[=rstan_config]{rstan_config()}} whenever
\code{stanmodel} files in \code{inst/stan} are added, removed, or modified.
}
\section{Using the pre-compiled Stan programs in your package}{
 The
\code{stanmodel} objects corresponding to the Stan programs included with your
package are stored in a list called \code{stanmodels}. To run one of the Stan
programs from within an R function in your package just pass the
appropriate element of the \code{stanmodels} list to one of the \pkg{rstan}
functions for model fitting (e.g., \code{sampling()}). For example, for a Stan
program \code{"foo.stan"} you would use \code{rstan::sampling(stanmodels$foo, ...)}.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/rstan_config.R
\name{rstan_config}
\alias{rstan_config}
\title{Configure system files for compiling Stan source code}
\usage{
rstan_config(pkgdir = ".")
}
\arguments{
\item{pkgdir}{Path to package root folder.}
}
\value{
Invisibly, whether or not any files were added/removed/modified by
the function.
}
\description{
Creates or update package-specific system files to compile \code{.stan} model
files found in \code{inst/stan}.
}
\details{
The Stan source files for the package should be stored in:
\itemize{
\item \code{inst/stan} for \code{.stan} files containing instructions to
build a \code{stanmodel} object.
\item \code{inst/stan/any_subfolder} for files to be included via the
\verb{#include "/my_subfolder/mylib.stan"} directive.
\item \code{inst/stan/any_subfolder} for a \code{license.stan} file.
\item \code{inst/include} for the \code{stan_meta_header.hpp} file, to be
used for directly interacting with the Stan C++ libraries.
}
}
